from typing import (
    IO,
    Any,
    Callable,
    Dict,
    Generator,
    Iterable,
    List,
    NoReturn,
    Pattern,
    Protocol,
    Sequence,
    Text,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

_T = TypeVar("_T")
_ActionT = TypeVar("_ActionT", bound=Action)
_N = TypeVar("_N")

_Text = Union[str, unicode]

ONE_OR_MORE: str
OPTIONAL: str
PARSER: str
REMAINDER: str
SUPPRESS: str
ZERO_OR_MORE: str
_UNRECOGNIZED_ARGS_ATTR: str  # undocumented

class ArgumentError(Exception):
    argument_name: str | None
    message: str
    def __init__(self, argument: Action | None, message: str) -> None: ...

# undocumented
class _AttributeHolder:
    def _get_kwargs(self) -> List[Tuple[str, Any]]: ...
    def _get_args(self) -> List[Any]: ...

# undocumented
class _ActionsContainer:
    description: _Text | None
    prefix_chars: _Text
    argument_default: Any
    conflict_handler: _Text

    _registries: Dict[_Text, Dict[Any, Any]]
    _actions: List[Action]
    _option_string_actions: Dict[_Text, Action]
    _action_groups: List[_ArgumentGroup]
    _mutually_exclusive_groups: List[_MutuallyExclusiveGroup]
    _defaults: Dict[str, Any]
    _negative_number_matcher: Pattern[str]
    _has_negative_number_optionals: List[bool]
    def __init__(self, description: Text | None, prefix_chars: Text, argument_default: Any, conflict_handler: Text) -> None: ...
    def register(self, registry_name: Text, value: Any, object: Any) -> None: ...
    def _registry_get(self, registry_name: Text, value: Any, default: Any = ...) -> Any: ...
    def set_defaults(self, **kwargs: Any) -> None: ...
    def get_default(self, dest: Text) -> Any: ...
    def add_argument(
        self,
        *name_or_flags: Text,
        action: Text | Type[Action] = ...,
        nargs: int | Text = ...,
        const: Any = ...,
        default: Any = ...,
        type: Callable[[Text], _T] | Callable[[str], _T] | FileType = ...,
        choices: Iterable[_T] = ...,
        required: bool = ...,
        help: Text | None = ...,
        metavar: Text | Tuple[Text, ...] | None = ...,
        dest: Text | None = ...,
        version: Text = ...,
        **kwargs: Any,
    ) -> Action: ...
    def add_argument_group(self, *args: Any, **kwargs: Any) -> _ArgumentGroup: ...
    def add_mutually_exclusive_group(self, **kwargs: Any) -> _MutuallyExclusiveGroup: ...
    def _add_action(self, action: _ActionT) -> _ActionT: ...
    def _remove_action(self, action: Action) -> None: ...
    def _add_container_actions(self, container: _ActionsContainer) -> None: ...
    def _get_positional_kwargs(self, dest: Text, **kwargs: Any) -> Dict[str, Any]: ...
    def _get_optional_kwargs(self, *args: Any, **kwargs: Any) -> Dict[str, Any]: ...
    def _pop_action_class(self, kwargs: Any, default: Type[Action] | None = ...) -> Type[Action]: ...
    def _get_handler(self) -> Callable[[Action, Iterable[Tuple[Text, Action]]], Any]: ...
    def _check_conflict(self, action: Action) -> None: ...
    def _handle_conflict_error(self, action: Action, conflicting_actions: Iterable[Tuple[Text, Action]]) -> NoReturn: ...
    def _handle_conflict_resolve(self, action: Action, conflicting_actions: Iterable[Tuple[Text, Action]]) -> None: ...

class _FormatterClass(Protocol):
    def __call__(self, prog: str) -> HelpFormatter: ...

class ArgumentParser(_AttributeHolder, _ActionsContainer):
    prog: _Text
    usage: _Text | None
    epilog: _Text | None
    formatter_class: _FormatterClass
    fromfile_prefix_chars: _Text | None
    add_help: bool

    # undocumented
    _positionals: _ArgumentGroup
    _optionals: _ArgumentGroup
    _subparsers: _ArgumentGroup | None
    def __init__(
        self,
        prog: Text | None = ...,
        usage: Text | None = ...,
        description: Text | None = ...,
        epilog: Text | None = ...,
        parents: Sequence[ArgumentParser] = ...,
        formatter_class: _FormatterClass = ...,
        prefix_chars: Text = ...,
        fromfile_prefix_chars: Text | None = ...,
        argument_default: Any = ...,
        conflict_handler: Text = ...,
        add_help: bool = ...,
    ) -> None: ...
    # The type-ignores in these overloads should be temporary.  See:
    # https://github.com/python/typeshed/pull/2643#issuecomment-442280277
    @overload
    def parse_args(self, args: Sequence[Text] | None = ...) -> Namespace: ...
    @overload
    def parse_args(self, args: Sequence[Text] | None, namespace: None) -> Namespace: ...  # type: ignore
    @overload
    def parse_args(self, args: Sequence[Text] | None, namespace: _N) -> _N: ...
    @overload
    def parse_args(self, *, namespace: None) -> Namespace: ...  # type: ignore
    @overload
    def parse_args(self, *, namespace: _N) -> _N: ...
    def add_subparsers(
        self,
        *,
        title: Text = ...,
        description: Text | None = ...,
        prog: Text = ...,
        parser_class: Type[ArgumentParser] = ...,
        action: Type[Action] = ...,
        option_string: Text = ...,
        dest: Text | None = ...,
        help: Text | None = ...,
        metavar: Text | None = ...,
    ) -> _SubParsersAction: ...
    def print_usage(self, file: IO[str] | None = ...) -> None: ...
    def print_help(self, file: IO[str] | None = ...) -> None: ...
    def format_usage(self) -> str: ...
    def format_help(self) -> str: ...
    def parse_known_args(
        self, args: Sequence[Text] | None = ..., namespace: Namespace | None = ...
    ) -> Tuple[Namespace, List[str]]: ...
    def convert_arg_line_to_args(self, arg_line: Text) -> List[str]: ...
    def exit(self, status: int = ..., message: Text | None = ...) -> NoReturn: ...
    def error(self, message: Text) -> NoReturn: ...
    # undocumented
    def _get_optional_actions(self) -> List[Action]: ...
    def _get_positional_actions(self) -> List[Action]: ...
    def _parse_known_args(self, arg_strings: List[Text], namespace: Namespace) -> Tuple[Namespace, List[str]]: ...
    def _read_args_from_files(self, arg_strings: List[Text]) -> List[Text]: ...
    def _match_argument(self, action: Action, arg_strings_pattern: Text) -> int: ...
    def _match_arguments_partial(self, actions: Sequence[Action], arg_strings_pattern: Text) -> List[int]: ...
    def _parse_optional(self, arg_string: Text) -> Tuple[Action | None, Text, Text | None] | None: ...
    def _get_option_tuples(self, option_string: Text) -> List[Tuple[Action, Text, Text | None]]: ...
    def _get_nargs_pattern(self, action: Action) -> _Text: ...
    def _get_values(self, action: Action, arg_strings: List[Text]) -> Any: ...
    def _get_value(self, action: Action, arg_string: Text) -> Any: ...
    def _check_value(self, action: Action, value: Any) -> None: ...
    def _get_formatter(self) -> HelpFormatter: ...
    def _print_message(self, message: str, file: IO[str] | None = ...) -> None: ...

class HelpFormatter:
    # undocumented
    _prog: _Text
    _indent_increment: int
    _max_help_position: int
    _width: int
    _current_indent: int
    _level: int
    _action_max_length: int
    _root_section: Any
    _current_section: Any
    _whitespace_matcher: Pattern[str]
    _long_break_matcher: Pattern[str]
    _Section: Type[Any]  # Nested class
    def __init__(
        self, prog: Text, indent_increment: int = ..., max_help_position: int = ..., width: int | None = ...
    ) -> None: ...
    def _indent(self) -> None: ...
    def _dedent(self) -> None: ...
    def _add_item(self, func: Callable[..., _Text], args: Iterable[Any]) -> None: ...
    def start_section(self, heading: Text | None) -> None: ...
    def end_section(self) -> None: ...
    def add_text(self, text: Text | None) -> None: ...
    def add_usage(
        self, usage: Text | None, actions: Iterable[Action], groups: Iterable[_ArgumentGroup], prefix: Text | None = ...
    ) -> None: ...
    def add_argument(self, action: Action) -> None: ...
    def add_arguments(self, actions: Iterable[Action]) -> None: ...
    def format_help(self) -> _Text: ...
    def _join_parts(self, part_strings: Iterable[Text]) -> _Text: ...
    def _format_usage(
        self, usage: Text, actions: Iterable[Action], groups: Iterable[_ArgumentGroup], prefix: Text | None
    ) -> _Text: ...
    def _format_actions_usage(self, actions: Iterable[Action], groups: Iterable[_ArgumentGroup]) -> _Text: ...
    def _format_text(self, text: Text) -> _Text: ...
    def _format_action(self, action: Action) -> _Text: ...
    def _format_action_invocation(self, action: Action) -> _Text: ...
    def _metavar_formatter(self, action: Action, default_metavar: Text) -> Callable[[int], Tuple[_Text, ...]]: ...
    def _format_args(self, action: Action, default_metavar: Text) -> _Text: ...
    def _expand_help(self, action: Action) -> _Text: ...
    def _iter_indented_subactions(self, action: Action) -> Generator[Action, None, None]: ...
    def _split_lines(self, text: Text, width: int) -> List[_Text]: ...
    def _fill_text(self, text: Text, width: int, indent: Text) -> _Text: ...
    def _get_help_string(self, action: Action) -> _Text | None: ...
    def _get_default_metavar_for_optional(self, action: Action) -> _Text: ...
    def _get_default_metavar_for_positional(self, action: Action) -> _Text: ...

class RawDescriptionHelpFormatter(HelpFormatter): ...
class RawTextHelpFormatter(RawDescriptionHelpFormatter): ...
class ArgumentDefaultsHelpFormatter(HelpFormatter): ...

class Action(_AttributeHolder):
    option_strings: Sequence[_Text]
    dest: _Text
    nargs: int | _Text | None
    const: Any
    default: Any
    type: Callable[[str], Any] | FileType | None
    choices: Iterable[Any] | None
    required: bool
    help: _Text | None
    metavar: _Text | Tuple[_Text, ...] | None
    def __init__(
        self,
        option_strings: Sequence[Text],
        dest: Text,
        nargs: int | Text | None = ...,
        const: _T | None = ...,
        default: _T | str | None = ...,
        type: Callable[[Text], _T] | Callable[[str], _T] | FileType | None = ...,
        choices: Iterable[_T] | None = ...,
        required: bool = ...,
        help: Text | None = ...,
        metavar: Text | Tuple[Text, ...] | None = ...,
    ) -> None: ...
    def __call__(
        self, parser: ArgumentParser, namespace: Namespace, values: Text | Sequence[Any] | None, option_string: Text | None = ...
    ) -> None: ...

class Namespace(_AttributeHolder):
    def __init__(self, **kwargs: Any) -> None: ...
    def __getattr__(self, name: Text) -> Any: ...
    def __setattr__(self, name: Text, value: Any) -> None: ...
    def __contains__(self, key: str) -> bool: ...

class FileType:
    # undocumented
    _mode: _Text
    _bufsize: int
    def __init__(self, mode: Text = ..., bufsize: int | None = ...) -> None: ...
    def __call__(self, string: Text) -> IO[Any]: ...

# undocumented
class _ArgumentGroup(_ActionsContainer):
    title: _Text | None
    _group_actions: List[Action]
    def __init__(
        self, container: _ActionsContainer, title: Text | None = ..., description: Text | None = ..., **kwargs: Any
    ) -> None: ...

# undocumented
class _MutuallyExclusiveGroup(_ArgumentGroup):
    required: bool
    _container: _ActionsContainer
    def __init__(self, container: _ActionsContainer, required: bool = ...) -> None: ...

# undocumented
class _StoreAction(Action): ...

# undocumented
class _StoreConstAction(Action):
    def __init__(
        self,
        option_strings: Sequence[Text],
        dest: Text,
        const: Any,
        default: Any = ...,
        required: bool = ...,
        help: Text | None = ...,
        metavar: Text | Tuple[Text, ...] | None = ...,
    ) -> None: ...

# undocumented
class _StoreTrueAction(_StoreConstAction):
    def __init__(
        self, option_strings: Sequence[Text], dest: Text, default: bool = ..., required: bool = ..., help: Text | None = ...
    ) -> None: ...

# undocumented
class _StoreFalseAction(_StoreConstAction):
    def __init__(
        self, option_strings: Sequence[Text], dest: Text, default: bool = ..., required: bool = ..., help: Text | None = ...
    ) -> None: ...

# undocumented
class _AppendAction(Action): ...

# undocumented
class _AppendConstAction(Action):
    def __init__(
        self,
        option_strings: Sequence[Text],
        dest: Text,
        const: Any,
        default: Any = ...,
        required: bool = ...,
        help: Text | None = ...,
        metavar: Text | Tuple[Text, ...] | None = ...,
    ) -> None: ...

# undocumented
class _CountAction(Action):
    def __init__(
        self, option_strings: Sequence[Text], dest: Text, default: Any = ..., required: bool = ..., help: Text | None = ...
    ) -> None: ...

# undocumented
class _HelpAction(Action):
    def __init__(
        self, option_strings: Sequence[Text], dest: Text = ..., default: Text = ..., help: Text | None = ...
    ) -> None: ...

# undocumented
class _VersionAction(Action):
    version: _Text | None
    def __init__(
        self, option_strings: Sequence[Text], version: Text | None = ..., dest: Text = ..., default: Text = ..., help: Text = ...
    ) -> None: ...

# undocumented
class _SubParsersAction(Action):
    _ChoicesPseudoAction: Type[Any]  # nested class
    _prog_prefix: _Text
    _parser_class: Type[ArgumentParser]
    _name_parser_map: Dict[_Text, ArgumentParser]
    choices: Dict[_Text, ArgumentParser]
    _choices_actions: List[Action]
    def __init__(
        self,
        option_strings: Sequence[Text],
        prog: Text,
        parser_class: Type[ArgumentParser],
        dest: Text = ...,
        help: Text | None = ...,
        metavar: Text | Tuple[Text, ...] | None = ...,
    ) -> None: ...
    # TODO: Type keyword args properly.
    def add_parser(self, name: Text, **kwargs: Any) -> ArgumentParser: ...
    def _get_subactions(self) -> List[Action]: ...

# undocumented
class ArgumentTypeError(Exception): ...

# undocumented
def _ensure_value(namespace: Namespace, name: Text, value: Any) -> Any: ...

# undocumented
def _get_action_name(argument: Action | None) -> str | None: ...



from email.mime.nonmultipart import MIMENonMultipart
from email.policy import Policy

class MIMEText(MIMENonMultipart):
    def __init__(self, _text: str, _subtype: str = ..., _charset: str | None = ..., *, policy: Policy | None = ...) -> None: ...



from typing import Any

from selenium.webdriver.remote.webelement import WebElement as WebElement

from . import interaction as interaction
from .interaction import Interaction as Interaction
from .mouse_button import MouseButton as MouseButton
from .pointer_input import PointerInput as PointerInput

class PointerActions(Interaction):
    source: Any
    def __init__(self, source: Any | None = ...) -> None: ...
    def pointer_down(self, button=...) -> None: ...
    def pointer_up(self, button=...) -> None: ...
    def move_to(self, element, x: Any | None = ..., y: Any | None = ...): ...
    def move_by(self, x, y): ...
    def move_to_location(self, x, y): ...
    def click(self, element: Any | None = ...): ...
    def context_click(self, element: Any | None = ...): ...
    def click_and_hold(self, element: Any | None = ...): ...
    def release(self): ...
    def double_click(self, element: Any | None = ...) -> None: ...
    def pause(self, duration: int = ...): ...



import existing_import
from existing_import import d
from m1 import A
from m2 import B
import m3
from m4 import D
from m5.sub import E
from m6 import F
from mStar import *
from m7 import a
from m8 import b
from ......m9 import c

def f(a1, a2, a3, a4, a5, a6):
    # type: (A, B, m3.C, D, E, F) -> G
    pass

def g(a7, a8, a9, a10):
    # type: (a, b, c, d) -> existing_import
    pass



from typing import Text

from .en import (
    EMOJI_ALIAS_UNICODE_ENGLISH as EMOJI_ALIAS_UNICODE_ENGLISH,
    EMOJI_UNICODE_ENGLISH as EMOJI_UNICODE_ENGLISH,
    UNICODE_EMOJI_ALIAS_ENGLISH as UNICODE_EMOJI_ALIAS_ENGLISH,
    UNICODE_EMOJI_ENGLISH as UNICODE_EMOJI_ENGLISH,
)
from .es import EMOJI_UNICODE_SPANISH as EMOJI_UNICODE_SPANISH, UNICODE_EMOJI_SPANISH as UNICODE_EMOJI_SPANISH
from .it import EMOJI_UNICODE_ITALIAN as EMOJI_UNICODE_ITALIAN, UNICODE_EMOJI_ITALIAN as UNICODE_EMOJI_ITALIAN
from .pt import EMOJI_UNICODE_PORTUGUESE as EMOJI_UNICODE_PORTUGUESE, UNICODE_EMOJI_PORTUGUESE as UNICODE_EMOJI_PORTUGUESE

EMOJI_UNICODE: dict[str, dict[Text, Text]]
UNICODE_EMOJI: dict[str, dict[Text, Text]]



import abc
import sys
from typing import (
    TYPE_CHECKING as TYPE_CHECKING,
    Any,
    AsyncContextManager as AsyncContextManager,
    AsyncGenerator as AsyncGenerator,
    AsyncIterable as AsyncIterable,
    AsyncIterator as AsyncIterator,
    Awaitable as Awaitable,
    Callable,
    ChainMap as ChainMap,
    ClassVar as ClassVar,
    ContextManager as ContextManager,
    Coroutine as Coroutine,
    Counter as Counter,
    DefaultDict as DefaultDict,
    Deque as Deque,
    ItemsView,
    KeysView,
    Mapping,
    NewType as NewType,
    NoReturn as NoReturn,
    Text as Text,
    Tuple,
    Type as Type,
    TypeVar,
    ValuesView,
    _Alias,
    overload as overload,
)

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., Any])
_TC = TypeVar("_TC", bound=Type[object])

class _SpecialForm:
    def __getitem__(self, typeargs: Any) -> Any: ...

def runtime_checkable(cls: _TC) -> _TC: ...

# This alias for above is kept here for backwards compatibility.
runtime = runtime_checkable
Protocol: _SpecialForm = ...
Final: _SpecialForm = ...
Self: _SpecialForm = ...
Required: _SpecialForm = ...
NotRequired: _SpecialForm = ...

def final(f: _F) -> _F: ...

Literal: _SpecialForm = ...

def IntVar(name: str) -> Any: ...  # returns a new TypeVar

if sys.version_info < (3, 8):
    # Technically in 3.6 this inherited from GenericMeta. But let's not reflect that, since
    # type checkers tend to assume that Protocols all have the ABCMeta metaclass.
    class _ProtocolMeta(abc.ABCMeta): ...

# Internal mypy fallback type for all typed dicts (does not exist at runtime)
class _TypedDict(Mapping[str, object], metaclass=abc.ABCMeta):
    __required_keys__: frozenset[str]
    __optional_keys__: frozenset[str]
    __total__: bool
    def copy(self: _T) -> _T: ...
    # Using NoReturn so that only calls using mypy plugin hook that specialize the signature
    # can go through.
    def setdefault(self, k: NoReturn, default: object) -> object: ...
    # Mypy plugin hook for 'pop' expects that 'default' has a type variable type.
    def pop(self, k: NoReturn, default: _T = ...) -> object: ...  # type: ignore
    def update(self: _T, __m: _T) -> None: ...
    def items(self) -> ItemsView[str, object]: ...
    def keys(self) -> KeysView[str]: ...
    def values(self) -> ValuesView[object]: ...
    def __delitem__(self, k: NoReturn) -> None: ...

# TypedDict is a (non-subscriptable) special form.
TypedDict: object = ...

OrderedDict = _Alias()

if sys.version_info >= (3, 7):
    def get_type_hints(
        obj: Callable[..., Any],
        globalns: dict[str, Any] | None = ...,
        localns: dict[str, Any] | None = ...,
        include_extras: bool = ...,
    ) -> dict[str, Any]: ...
    def get_args(tp: Any) -> Tuple[Any, ...]: ...
    def get_origin(tp: Any) -> Any | None: ...

Annotated: _SpecialForm = ...
_AnnotatedAlias: Any = ...  # undocumented

@runtime_checkable
class SupportsIndex(Protocol, metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __index__(self) -> int: ...

# PEP 612 support for Python < 3.9
if sys.version_info >= (3, 10):
    from typing import Concatenate as Concatenate, ParamSpec as ParamSpec, TypeAlias as TypeAlias, TypeGuard as TypeGuard
else:
    class ParamSpecArgs:
        __origin__: ParamSpec
        def __init__(self, origin: ParamSpec) -> None: ...
    class ParamSpecKwargs:
        __origin__: ParamSpec
        def __init__(self, origin: ParamSpec) -> None: ...
    class ParamSpec:
        __name__: str
        __bound__: Type[Any] | None
        __covariant__: bool
        __contravariant__: bool
        def __init__(
            self, name: str, *, bound: None | Type[Any] | str = ..., contravariant: bool = ..., covariant: bool = ...
        ) -> None: ...
        @property
        def args(self) -> ParamSpecArgs: ...
        @property
        def kwargs(self) -> ParamSpecKwargs: ...
    Concatenate: _SpecialForm = ...
    TypeAlias: _SpecialForm = ...
    TypeGuard: _SpecialForm = ...



from sre_constants import (
    SRE_FLAG_ASCII as SRE_FLAG_ASCII,
    SRE_FLAG_DEBUG as SRE_FLAG_DEBUG,
    SRE_FLAG_DOTALL as SRE_FLAG_DOTALL,
    SRE_FLAG_IGNORECASE as SRE_FLAG_IGNORECASE,
    SRE_FLAG_LOCALE as SRE_FLAG_LOCALE,
    SRE_FLAG_MULTILINE as SRE_FLAG_MULTILINE,
    SRE_FLAG_TEMPLATE as SRE_FLAG_TEMPLATE,
    SRE_FLAG_UNICODE as SRE_FLAG_UNICODE,
    SRE_FLAG_VERBOSE as SRE_FLAG_VERBOSE,
    SRE_INFO_CHARSET as SRE_INFO_CHARSET,
    SRE_INFO_LITERAL as SRE_INFO_LITERAL,
    SRE_INFO_PREFIX as SRE_INFO_PREFIX,
    _NamedIntConstant,
)
from sre_parse import SubPattern
from typing import Any, Pattern

MAXCODE: int

def dis(code: list[_NamedIntConstant]) -> None: ...
def isstring(obj: Any) -> bool: ...
def compile(p: str | bytes | SubPattern, flags: int = ...) -> Pattern[Any]: ...



LEXER_ENTRY_POINT: str
FORMATTER_ENTRY_POINT: str
STYLE_ENTRY_POINT: str
FILTER_ENTRY_POINT: str

def iter_entry_points(group_name): ...
def find_plugin_lexers() -> None: ...
def find_plugin_formatters() -> None: ...
def find_plugin_styles() -> None: ...
def find_plugin_filters() -> None: ...



from _typeshed import Self
from types import ModuleType
from typing import Any, Callable, Generic, TypeVar, overload
from unittest import TestLoader, TestSuite

from setuptools import Command

_T = TypeVar("_T")

class ScanningLoader(TestLoader):
    def __init__(self) -> None: ...
    def loadTestsFromModule(self, module: ModuleType, pattern: Any | None = ...) -> list[TestSuite]: ...  # type: ignore[override]

class NonDataProperty(Generic[_T]):
    fget: Callable[..., _T]
    def __init__(self, fget: Callable[..., _T]) -> None: ...
    @overload
    def __get__(self: Self, obj: None, objtype: object = ...) -> Self: ...
    @overload
    def __get__(self, obj: Any, objtype: object = ...) -> _T: ...

class test(Command):
    description: str
    user_options: Any
    test_suite: Any
    test_module: Any
    test_loader: Any
    test_runner: Any
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    # TODO: uncomment once https://github.com/python/mypy/pull/10884 is released
    # def test_args(self): ...
    def with_project_on_sys_path(self, func) -> None: ...
    def project_on_sys_path(self, include_dists=...): ...
    @staticmethod
    def paths_on_pythonpath(paths) -> None: ...
    @staticmethod
    def install_dists(dist): ...
    def run(self) -> None: ...
    def run_tests(self) -> None: ...



from typing import Any

PY2: Any
text_type = str
string_types: Any
integer_types: Any
text_to_native: Any
unichr = chr
iterkeys: Any
itervalues: Any
iteritems: Any
izip = zip
imap = map
range_type = range
cmp: Any
array_tobytes: Any
number_types: Any

def force_text(s, encoding: str = ..., errors: str = ...): ...



from typing import Any

def compact(text, *args, **kw): ...
def compact_empty_lines(text): ...
def concatenate(items, conjunction: str = ..., serial_comma: bool = ...): ...
def dedent(text, *args, **kw): ...
def format(text, *args, **kw): ...
def generate_slug(text, delimiter: str = ...): ...
def is_empty_line(text): ...
def join_lines(text): ...
def pluralize(count, singular, plural: Any | None = ...): ...
def pluralize_raw(count, singular, plural: Any | None = ...): ...
def random_string(length=..., characters=...): ...
def split(text, delimiter: str = ...): ...
def split_paragraphs(text): ...
def tokenize(text): ...
def trim_empty_lines(text): ...



"""Tests for load_pytd.py."""

import collections
import contextlib
import io
import os
import textwrap

from pytype import config
from pytype import file_utils
from pytype import load_pytd
from pytype import module_utils
from pytype.pytd import pytd
from pytype.pytd import pytd_utils
from pytype.pytd import serialize_ast
from pytype.pytd import visitors
from pytype.tests import test_base

import unittest


class _LoaderTest(test_base.UnitTest):

  @contextlib.contextmanager
  def _setup_loader(self, **kwargs):
    with file_utils.Tempdir() as d:
      for name, contents in kwargs.items():
        d.create_file(f"{name}.pyi", contents)
      yield load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))

  def _import(self, **kwargs):
    with self._setup_loader(**kwargs) as loader:
      return loader.import_name(kwargs.popitem()[0])


class ImportPathsTest(_LoaderTest):
  """Tests for load_pytd.py."""

  def test_filepath_to_module(self):
    # (filename, pythonpath, expected)
    test_cases = [
        ("foo/bar/baz.py", [""], "foo.bar.baz"),
        ("foo/bar/baz.py", ["foo"], "bar.baz"),
        ("foo/bar/baz.py", ["fo"], "foo.bar.baz"),
        ("foo/bar/baz.py", ["foo/"], "bar.baz"),
        ("foo/bar/baz.py", ["foo", "bar"], "bar.baz"),
        ("foo/bar/baz.py", ["foo/bar", "foo"], "baz"),
        ("foo/bar/baz.py", ["foo", "foo/bar"], "bar.baz"),
        ("./foo/bar.py", [""], "foo.bar"),
        ("./foo.py", [""], "foo"),
        ("../foo.py", [""], None),
        ("../foo.py", ["."], None),
        ("foo/bar/../baz.py", [""], "foo.baz"),
        ("../foo.py", [".."], "foo"),
        ("../../foo.py", ["../.."], "foo"),
        ("../../foo.py", [".."], None)
    ]
    for filename, pythonpath, expected in test_cases:
      module = module_utils.get_module_name(filename, pythonpath)
      self.assertEqual(module, expected)

  def test_builtin_sys(self):
    loader = load_pytd.Loader(config.Options.create(
        module_name="base", python_version=self.python_version))
    ast = loader.import_name("sys")
    self.assertTrue(ast.Lookup("sys.exit"))

  def test_basic(self):
    with file_utils.Tempdir() as d:
      d.create_file("path/to/some/module.pyi", "def foo(x:int) -> str: ...")
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      ast = loader.import_name("path.to.some.module")
      self.assertTrue(ast.Lookup("path.to.some.module.foo"))

  def test_path(self):
    with file_utils.Tempdir() as d1:
      with file_utils.Tempdir() as d2:
        d1.create_file("dir1/module1.pyi", "def foo1() -> str: ...")
        d2.create_file("dir2/module2.pyi", "def foo2() -> str: ...")
        loader = load_pytd.Loader(config.Options.create(
            module_name="base", python_version=self.python_version,
            pythonpath=f"{d1.path}:{d2.path}"))
        module1 = loader.import_name("dir1.module1")
        module2 = loader.import_name("dir2.module2")
        self.assertTrue(module1.Lookup("dir1.module1.foo1"))
        self.assertTrue(module2.Lookup("dir2.module2.foo2"))

  def test_init(self):
    with file_utils.Tempdir() as d1:
      d1.create_file("baz/__init__.pyi", "x = ... # type: int")
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d1.path))
      self.assertTrue(loader.import_name("baz").Lookup("baz.x"))

  def test_builtins(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", "x = ... # type: int")
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      mod = loader.import_name("foo")
      self.assertEqual("builtins.int", mod.Lookup("foo.x").type.cls.name)
      self.assertEqual("builtins.int", mod.Lookup("foo.x").type.name)

  def test_no_init(self):
    with file_utils.Tempdir() as d:
      d.create_directory("baz")
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      self.assertTrue(loader.import_name("baz"))

  def test_no_init_imports_map(self):
    with file_utils.Tempdir() as d:
      d.create_directory("baz")
      os.chdir(d.path)
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=""))
      loader.options.tweak(imports_map={})
      self.assertFalse(loader.import_name("baz"))

  def test_stdlib(self):
    loader = load_pytd.Loader(config.Options.create(
        module_name="base", python_version=self.python_version))
    ast = loader.import_name("io")
    self.assertTrue(ast.Lookup("io.StringIO"))

  def test_deep_dependency(self):
    with file_utils.Tempdir() as d:
      d.create_file("module1.pyi", "def get_bar() -> module2.Bar: ...")
      d.create_file("module2.pyi", "class Bar:\n  pass")
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      module1 = loader.import_name("module1")
      f, = module1.Lookup("module1.get_bar").signatures
      self.assertEqual("module2.Bar", f.return_type.cls.name)

  def test_circular_dependency(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", """
        def get_bar() -> bar.Bar: ...
        class Foo:
          pass
      """)
      d.create_file("bar.pyi", """
        def get_foo() -> foo.Foo: ...
        class Bar:
          pass
      """)
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      foo = loader.import_name("foo")
      bar = loader.import_name("bar")
      f1, = foo.Lookup("foo.get_bar").signatures
      f2, = bar.Lookup("bar.get_foo").signatures
      self.assertEqual("bar.Bar", f1.return_type.cls.name)
      self.assertEqual("foo.Foo", f2.return_type.cls.name)

  def test_circular_dependency_complicated(self):
    # The dependency graph looks like:
    # target ----------
    # |               |
    # v               v
    # dep1 -> dep2 -> dep3
    # ^               |
    # |               |
    # -----------------
    with self._setup_loader(target="""
      from dep1 import PathLike
      from dep3 import AnyPath
      def abspath(path: PathLike[str]) -> str: ...
    """, dep1="""
      from dep2 import Popen
      from typing import Generic, TypeVar
      _T = TypeVar('_T')
      class PathLike(Generic[_T]): ...
    """, dep2="""
      from dep3 import AnyPath
      class Popen: ...
    """, dep3="""
      from dep1 import PathLike
      AnyPath = PathLike[str]
    """) as loader:
      loader.finish_and_verify_ast(
          loader.load_file("target", os.path.join(
              loader.options.pythonpath[0], "target.pyi")))

  def test_relative(self):
    with file_utils.Tempdir() as d:
      d.create_file("__init__.pyi", "base = ...  # type: str")
      d.create_file("path/__init__.pyi", "path = ...  # type: str")
      d.create_file("path/to/__init__.pyi", "to = ...  # type: str")
      d.create_file("path/to/some/__init__.pyi", "some = ...  # type: str")
      d.create_file("path/to/some/module.pyi", "")
      loader = load_pytd.Loader(config.Options.create(
          module_name="path.to.some.module", python_version=self.python_version,
          pythonpath=d.path))
      some = loader.import_relative(1)
      to = loader.import_relative(2)
      path = loader.import_relative(3)
      # Python doesn't allow "...." here, so don't test import_relative(4).
      self.assertTrue(some.Lookup("path.to.some.some"))
      self.assertTrue(to.Lookup("path.to.to"))
      self.assertTrue(path.Lookup("path.path"))

  def test_typeshed(self):
    loader = load_pytd.Loader(config.Options.create(
        module_name="base", python_version=self.python_version))
    self.assertTrue(loader.import_name("urllib.request"))

  def test_prefer_typeshed(self):
    with file_utils.Tempdir() as d:
      # Override two modules from typeshed
      d.create_file("typing_extensions/__init__.pyi", "foo: str = ...")
      d.create_file("crypt/__init__.pyi", "foo: str = ...")
      loader = load_pytd.Loader(config.Options.create(
          module_name="x", python_version=self.python_version,
          pythonpath=d.path))
      # typing_extensions should ignore the override, crypt should not.
      ast1 = loader.import_name("typing_extensions")
      ast2 = loader.import_name("crypt")
      self.assertTrue(ast1.Lookup("typing_extensions.Literal"))
      self.assertTrue(ast2.Lookup("crypt.foo"))
      with self.assertRaises(KeyError):
        ast1.Lookup("typing_extensions.foo")
      with self.assertRaises(KeyError):
        ast2.Lookup("crypt.crypt")

  def test_resolve_alias(self):
    with file_utils.Tempdir() as d:
      d.create_file("module1.pyi", """
          from typing import List
          x = List[int]
      """)
      d.create_file("module2.pyi", """
          def f() -> module1.x: ...
      """)
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=d.path))
      module2 = loader.import_name("module2")
      f, = module2.Lookup("module2.f").signatures
      self.assertEqual("List[int]", pytd_utils.Print(f.return_type))

  def test_import_map_congruence(self):
    with file_utils.Tempdir() as d:
      foo_path = d.create_file("foo.pyi", "class X: ...")
      bar_path = d.create_file("bar.pyi", "X = ...  # type: another.foo.X")
      # Map the same pyi file under two module paths.
      imports_map = {
          "foo": foo_path,
          "another/foo": foo_path,
          "bar": bar_path,
          "empty1": "/dev/null",
          "empty2": "/dev/null",
      }
      loader = load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version,
          pythonpath=""))
      loader.options.tweak(imports_map=imports_map)
      normal = loader.import_name("foo")
      self.assertEqual("foo", normal.name)
      loader.import_name("bar")  # check that we can resolve against another.foo
      another = loader.import_name("another.foo")
      # We do *not* treat foo.X and another.foo.X the same, because Python
      # doesn't, either:
      self.assertIsNot(normal, another)
      self.assertTrue([c.name.startswith("foo")
                       for c in normal.classes])
      self.assertTrue([c.name.startswith("another.foo")
                       for c in another.classes])
      # Make sure that multiple modules using /dev/null are not treated as
      # congruent.
      empty1 = loader.import_name("empty1")
      empty2 = loader.import_name("empty2")
      self.assertIsNot(empty1, empty2)
      self.assertEqual("empty1", empty1.name)
      self.assertEqual("empty2", empty2.name)

  def test_package_relative_import(self):
    with file_utils.Tempdir() as d:
      d.create_file("pkg/foo.pyi", "class X: ...")
      d.create_file("pkg/bar.pyi", """
          from .foo import X
          y = ...  # type: X""")
      loader = load_pytd.Loader(config.Options.create(
          module_name="pkg.bar", python_version=self.python_version,
          pythonpath=d.path))
      bar = loader.import_name("pkg.bar")
      f = bar.Lookup("pkg.bar.y")
      self.assertEqual("pkg.foo.X", f.type.name)

  def test_directory_import(self):
    with file_utils.Tempdir() as d:
      d.create_file("pkg/sub/__init__.pyi", """
          from .foo import *
          from .bar import *""")
      d.create_file("pkg/sub/foo.pyi", """
          class X: pass""")
      d.create_file("pkg/sub/bar.pyi", """
          from .foo import X
          y = ...  # type: X""")
      loader = load_pytd.Loader(config.Options.create(
          module_name="pkg", python_version=self.python_version,
          pythonpath=d.path))
      ast = loader.import_name("pkg.sub")
      self.assertTrue(ast.Lookup("pkg.sub.X"))

  def test_diamond_import(self):
    """Should not fail on importing a module via two paths."""
    with file_utils.Tempdir() as d:
      d.create_file("pkg/sub/__init__.pyi", """
          from .foo import *
          from .bar import *""")
      d.create_file("pkg/sub/foo.pyi", """
          from .baz import X""")
      d.create_file("pkg/sub/bar.pyi", """
          from .baz import X""")
      d.create_file("pkg/sub/baz.pyi", """
          class X: ...""")
      loader = load_pytd.Loader(config.Options.create(
          module_name="pkg", python_version=self.python_version,
          pythonpath=d.path))
      ast = loader.import_name("pkg.sub")
      self.assertTrue(ast.Lookup("pkg.sub.X"))

  def test_get_resolved_modules(self):
    with file_utils.Tempdir() as d:
      filename = d.create_file("dir/module.pyi", "def foo() -> str: ...")
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      ast = loader.import_name("dir.module")
      modules = loader.get_resolved_modules()
      self.assertEqual(set(modules), {"builtins", "typing", "dir.module"})
      module = modules["dir.module"]
      self.assertEqual(module.module_name, "dir.module")
      self.assertEqual(module.filename, filename)
      self.assertEqual(module.ast, ast)

  def test_circular_import(self):
    with file_utils.Tempdir() as d:
      d.create_file("os2/__init__.pyi", """
        from . import path as path
        _PathType = path._PathType
        def utime(path: _PathType) -> None: ...
        class stat_result: ...
      """)
      d.create_file("os2/path.pyi", """
        import os2
        _PathType = bytes
        def samestat(stat1: os2.stat_result) -> bool: ...
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      ast = loader.import_name("os2.path")
      self.assertEqual(ast.Lookup("os2.path._PathType").type.name,
                       "builtins.bytes")

  def test_circular_import_with_external_type(self):
    with file_utils.Tempdir() as d:
      d.create_file("os2/__init__.pyi", """
        from posix2 import stat_result as stat_result
        from . import path as path
        _PathType = path._PathType
        def utime(path: _PathType) -> None: ...
      """)
      d.create_file("os2/path.pyi", """
        import os2
        _PathType = bytes
        def samestate(stat1: os2.stat_result) -> bool: ...
      """)
      d.create_file("posix2.pyi", "class stat_result: ...")
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      # Make sure all three modules were resolved properly.
      loader.import_name("os2")
      loader.import_name("os2.path")
      loader.import_name("posix2")

  def test_union_alias(self):
    with file_utils.Tempdir() as d:
      d.create_file("test.pyi", """
        from typing import Union as _UnionT
        x: _UnionT[int, str]
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      ast = loader.import_name("test")
      x = ast.Lookup("test.x")
      self.assertIsInstance(x.type, pytd.UnionType)

  def test_optional_alias(self):
    with file_utils.Tempdir() as d:
      d.create_file("test.pyi", """
        from typing import Optional as _OptionalT
        x: _OptionalT[int]
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      ast = loader.import_name("test")
      x = ast.Lookup("test.x")
      self.assertIsInstance(x.type, pytd.UnionType)

  def test_intersection_alias(self):
    with file_utils.Tempdir() as d:
      d.create_file("test.pyi", """
        from typing import Intersection as _IntersectionT
        x: _IntersectionT[int, str]
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      ast = loader.import_name("test")
      x = ast.Lookup("test.x")
      self.assertIsInstance(x.type, pytd.IntersectionType)

  def test_open_function(self):
    def mock_open(*unused_args, **unused_kwargs):
      return io.StringIO("x: int")
    loader = load_pytd.Loader(config.Options.create(
        module_name="base", python_version=self.python_version,
        open_function=mock_open))
    a = loader.load_file("a", "a.pyi")
    self.assertEqual("int", pytd_utils.Print(a.Lookup("a.x").type))

  def test_submodule_reexport(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo/bar.pyi", "")
      d.create_file("foo/__init__.pyi", """
        from . import bar as bar
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      foo = loader.import_name("foo")
      self.assertEqual(pytd_utils.Print(foo), "import foo.bar")

  def test_submodule_rename(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo/bar.pyi", "")
      d.create_file("foo/__init__.pyi", """
        from . import bar as baz
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      foo = loader.import_name("foo")
      self.assertEqual(pytd_utils.Print(foo), "from foo import bar as foo.baz")

  def test_typing_reexport(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", """
        from typing import List as List
      """)
      d.create_file("bar.pyi", """
        from foo import *
        def f() -> List[int]: ...
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      foo = loader.import_name("foo")
      bar = loader.import_name("bar")
      self.assertEqual(pytd_utils.Print(foo),
                       "from builtins import list as List")
      self.assertEqual(pytd_utils.Print(bar), textwrap.dedent("""
        import typing
        from builtins import list as List

        def bar.f() -> typing.List[int]: ...
      """).strip())

  def test_reuse_builtin_name(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", """
        class Ellipsis: ...
      """)
      d.create_file("bar.pyi", """
        from foo import *
        def f(x: Ellipsis): ...
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      loader.import_name("foo")
      bar = loader.import_name("bar")
      self.assertEqual(pytd_utils.Print(bar.Lookup("bar.f")),
                       "def bar.f(x: foo.Ellipsis) -> Any: ...")

  def test_import_typevar(self):
    # Regression test for the loader crashing with a
    # ""Duplicate top level items: 'T', 'T'" error.
    self._import(a="""
      from typing import TypeVar
      T = TypeVar('T')
    """, b="""
      from a import T
      def f(x: T) -> T: ...
    """, c="""
      from b import *
    """)

  def test_import_class_from_parent_module(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo/__init__.pyi", "class Foo: ...")
      d.create_file("foo/bar.pyi", """
        from . import Foo
        class Bar(Foo): ...
      """)
      loader = load_pytd.Loader(config.Options.create(
          python_version=self.python_version, pythonpath=d.path))
      loader.import_name("foo.bar")

  def test_module_alias(self):
    ast = self._import(foo="""
      import subprocess as _subprocess
      x: _subprocess.Popen
    """)
    expected = textwrap.dedent("""
      import subprocess as foo._subprocess

      foo.x: _subprocess.Popen
    """).strip()
    self.assertMultiLineEqual(pytd_utils.Print(ast), expected)


class ImportTypeMacroTest(_LoaderTest):

  def test_container(self):
    ast = self._import(a="""
      from typing import List, TypeVar
      T = TypeVar('T')
      Alias = List[T]
    """, b="""
      import a
      Strings = a.Alias[str]
    """)
    self.assertEqual(
        pytd_utils.Print(ast.Lookup("b.Strings").type), "List[str]")

  def test_union(self):
    ast = self._import(a="""
      from typing import List, TypeVar, Union
      T = TypeVar('T')
      Alias = Union[T, List[T]]
    """, b="""
      import a
      Strings = a.Alias[str]
    """)
    self.assertEqual(pytd_utils.Print(ast.Lookup("b.Strings").type),
                     "Union[str, List[str]]")

  def test_bad_parameterization(self):
    with self.assertRaisesRegex(
        load_pytd.BadDependencyError,
        r"Union\[T, List\[T\]\] expected 1 parameters, got 2"):
      self._import(a="""
        from typing import List, TypeVar, Union
        T = TypeVar('T')
        Alias = Union[T, List[T]]
      """, b="""
        import a
        Strings = a.Alias[str, str]
      """)

  def test_no_parameters(self):
    ast = self._import(a="""
      from typing import List, TypeVar
      T = TypeVar('T')
      Alias = List[T]
    """, b="""
      import a
      def f(x: a.Alias): ...
    """)
    self.assertEqual(
        pytd_utils.Print(ast.Lookup("b.f").signatures[0].params[0].type),
        "List[Any]")


_Module = collections.namedtuple("_", ["module_name", "file_name"])


class PickledPyiLoaderTest(test_base.UnitTest):

  def _create_files(self, tempdir):
    src = """
        import module2
        from typing import List

        constant = True

        x = List[int]
        b = List[int]

        class SomeClass:
          def __init__(self, a: module2.ObjectMod2):
            pass

        def ModuleFunction():
          pass
    """
    tempdir.create_file("module1.pyi", src)
    tempdir.create_file("module2.pyi", """
        class ObjectMod2:
          def __init__(self):
            pass
    """)

  def _get_path(self, tempdir, filename):
    return os.path.join(tempdir.path, filename)

  def _load_ast(self, tempdir, module):
    loader = load_pytd.Loader(config.Options.create(
        module_name=module.module_name, python_version=self.python_version,
        pythonpath=tempdir.path))
    return loader, loader.load_file(
        module.module_name, self._get_path(tempdir, module.file_name))

  def _pickle_modules(self, loader, tempdir, *modules):
    for module in modules:
      serialize_ast.StoreAst(
          loader._modules[module.module_name].ast,
          self._get_path(tempdir, module.file_name + ".pickled"))

  def _load_pickled_module(self, tempdir, module):
    pickle_loader = load_pytd.PickledPyiLoader(config.Options.create(
        python_version=self.python_version, pythonpath=tempdir.path))
    return pickle_loader.load_file(
        module.module_name, self._get_path(tempdir, module.file_name))

  def test_load_with_same_module_name(self):
    with file_utils.Tempdir() as d:
      self._create_files(tempdir=d)
      module1 = _Module(module_name="foo.bar.module1", file_name="module1.pyi")
      module2 = _Module(module_name="module2", file_name="module2.pyi")
      loader, ast = self._load_ast(tempdir=d, module=module1)
      self._pickle_modules(loader, d, module1, module2)
      pickled_ast_filename = self._get_path(d, module1.file_name + ".pickled")
      result = serialize_ast.StoreAst(ast, pickled_ast_filename)
      self.assertIsNone(result)

      loaded_ast = self._load_pickled_module(d, module1)
      self.assertTrue(loaded_ast)
      self.assertIsNot(loaded_ast, ast)
      self.assertTrue(pytd_utils.ASTeq(ast, loaded_ast))
      loaded_ast.Visit(visitors.VerifyLookup())

  def test_star_import(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", "class A: ...")
      d.create_file("bar.pyi", "from foo import *")
      foo = _Module(module_name="foo", file_name="foo.pyi")
      bar = _Module(module_name="bar", file_name="bar.pyi")
      loader, _ = self._load_ast(d, module=bar)
      self._pickle_modules(loader, d, foo, bar)
      loaded_ast = self._load_pickled_module(d, bar)
      loaded_ast.Visit(visitors.VerifyLookup())
      self.assertEqual(pytd_utils.Print(loaded_ast), "from foo import A")

  def test_function_alias(self):
    with file_utils.Tempdir() as d:
      d.create_file("foo.pyi", """
        def f(): ...
        g = f
      """)
      foo = _Module(module_name="foo", file_name="foo.pyi")
      loader, _ = self._load_ast(d, module=foo)
      self._pickle_modules(loader, d, foo)
      loaded_ast = self._load_pickled_module(d, foo)
      g = loaded_ast.Lookup("foo.g")
      self.assertEqual(g.type, loaded_ast.Lookup("foo.f"))

  def test_package_relative_import(self):
    with file_utils.Tempdir() as d:
      d.create_file("pkg/foo.pyi", "class X: ...")
      d.create_file("pkg/bar.pyi", """
          from .foo import X
          y = ...  # type: X""")
      foo = _Module(module_name="pkg.foo", file_name="pkg/foo.pyi")
      bar = _Module(module_name="pkg.bar", file_name="pkg/bar.pyi")
      loader, _ = self._load_ast(d, module=bar)
      self._pickle_modules(loader, d, foo, bar)
      loaded_ast = self._load_pickled_module(d, bar)
      loaded_ast.Visit(visitors.VerifyLookup())

  def test_pickled_builtins(self):
    with file_utils.Tempdir() as d:
      filename = d.create_file("builtins.pickle")
      foo_path = d.create_file("foo.pickle", """
        import datetime
        tz = ...  # type: datetime.tzinfo
      """)
      # save builtins
      load_pytd.Loader(config.Options.create(
          module_name="base", python_version=self.python_version
      )).save_to_pickle(filename)
      # load builtins
      loader = load_pytd.PickledPyiLoader.load_from_pickle(
          filename,
          config.Options.create(
              module_name="base",
              python_version=self.python_version,
              pythonpath=""))
      loader.options.tweak(imports_map={"foo": foo_path})
      # test import
      self.assertTrue(loader.import_name("sys"))
      self.assertTrue(loader.import_name("__future__"))
      self.assertTrue(loader.import_name("datetime"))
      self.assertTrue(loader.import_name("foo"))
      self.assertTrue(loader.import_name("ctypes"))


class MethodAliasTest(_LoaderTest):

  def test_import_class(self):
    b_ast = self._import(a="""
      class Foo:
        def f(self) -> int: ...
    """, b="""
      import a
      f = a.Foo.f
    """)
    self.assertEqual(pytd_utils.Print(b_ast.Lookup("b.f")),
                     "def b.f(self: a.Foo) -> int: ...")

  def test_import_class_instance(self):
    b_ast = self._import(a="""
      class Foo:
        def f(self) -> int: ...
      foo: Foo
    """, b="""
      import a
      f = a.foo.f
    """)
    self.assertEqual(pytd_utils.Print(b_ast.Lookup("b.f")),
                     "def b.f() -> int: ...")

  def test_create_instance_after_import(self):
    b_ast = self._import(a="""
      class Foo:
        def f(self) -> int: ...
    """, b="""
      import a
      foo: a.Foo
      f = foo.f
    """)
    self.assertEqual(pytd_utils.Print(b_ast.Lookup("b.f")),
                     "def b.f() -> int: ...")

  def test_function(self):
    ast = self._import(a="""
      def f(x: int) -> int: ...
      g = f
    """)
    self.assertEqual(pytd_utils.Print(ast.Lookup("a.g")),
                     "def a.g(x: int) -> int: ...")

  def test_imported_function(self):
    b_ast = self._import(a="""
      def f(x: int) -> int: ...
    """, b="""
      import a
      f = a.f
    """)
    self.assertEqual(pytd_utils.Print(b_ast.Lookup("b.f")),
                     "def b.f(x: int) -> int: ...")

  def test_base_class(self):
    a_ast = self._import(a="""
      class Foo:
        def f(self) -> int: ...
      class Bar(Foo): ...
      x: Bar
      f = x.f
    """)
    self.assertEqual(pytd_utils.Print(a_ast.Lookup("a.f")),
                     "def a.f() -> int: ...")

  def test_base_class_imported(self):
    b_ast = self._import(a="""
      class Foo:
        def f(self) -> int: ...
      class Bar(Foo): ...
      x: Bar
    """, b="""
      import a
      f = a.x.f
    """)
    self.assertEqual(pytd_utils.Print(b_ast.Lookup("b.f")),
                     "def b.f() -> int: ...")


class RecursiveAliasTest(_LoaderTest):

  def test_basic(self):
    ast = self._import(a="""
      from typing import List
      X = List[X]
    """)
    actual_x = ast.Lookup("a.X")
    expected_x = pytd.Alias(
        name="a.X",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(pytd.LateType("a.X"),)))
    self.assertEqual(actual_x, expected_x)

  def test_mutual_recursion(self):
    ast = self._import(a="""
      from typing import List
      X = List[Y]
      Y = List[X]
    """)

    actual_x = ast.Lookup("a.X")
    expected_x = pytd.Alias(
        name="a.X",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(pytd.LateType("a.Y"),)))
    self.assertEqual(actual_x, expected_x)

    actual_y = ast.Lookup("a.Y")
    expected_y = pytd.Alias(
        name="a.Y",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(
                pytd.GenericType(
                    base_type=pytd.ClassType("builtins.list"),
                    parameters=(pytd.LateType("a.Y"),)),)))
    self.assertEqual(actual_y, expected_y)

  def test_very_mutual_recursion(self):
    ast = self._import(a="""
      from typing import List
      X = List[Y]
      Y = List[Z]
      Z = List[X]
    """)

    actual_x = ast.Lookup("a.X")
    expected_x = pytd.Alias(
        name="a.X",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(pytd.LateType("a.Y"),)))
    self.assertEqual(actual_x, expected_x)

    actual_y = ast.Lookup("a.Y")
    expected_y = pytd.Alias(
        name="a.Y",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(pytd.LateType("a.Z"),)))
    self.assertEqual(actual_y, expected_y)

    actual_z = ast.Lookup("a.Z")
    expected_z = pytd.Alias(
        name="a.Z",
        type=pytd.GenericType(
            base_type=pytd.ClassType("builtins.list"),
            parameters=(
                pytd.GenericType(
                    base_type=pytd.ClassType("builtins.list"),
                    parameters=(pytd.LateType("a.Y"),)),)))
    self.assertEqual(actual_z, expected_z)


class NestedClassTest(_LoaderTest):

  def test_basic(self):
    ast = self._import(a="""
      class A:
        class B:
          def f(self) -> C: ...
        class C(B): ...
    """)
    actual_f = pytd.LookupItemRecursive(ast, "A.B.f")
    self.assertEqual(pytd_utils.Print(actual_f),
                     "def f(self: a.A.B) -> a.A.C: ...")
    actual_c = pytd.LookupItemRecursive(ast, "A.C")
    self.assertEqual(pytd_utils.Print(actual_c).rstrip(),
                     "class a.A.C(a.A.B): ...")

  @test_base.skip("This does not work yet")
  def test_shadowing(self):
    ast = self._import(a="""
      class A:
        class A(A):
          def f(self) -> A: ...
        class C(A): ...
    """)
    self.assertEqual(pytd_utils.Print(ast).rstrip(), textwrap.dedent("""
      class a.A:
          class a.A.A(a.A):
              def f(self) -> a.A.A: ...
          class a.A.C(a.A.A): ...
      """).strip())


if __name__ == "__main__":
  unittest.main()



from typing import Any, NamedTuple

operators: Any
escapes: Any
name_re: Any
dotted_name_re: Any
division_re: Any
regex_re: Any
line_re: Any
line_join_re: Any
uni_escape_re: Any

class Token(NamedTuple):
    type: Any
    value: Any
    lineno: Any

def get_rules(jsx, dotted, template_string): ...
def indicates_division(token): ...
def unquote_string(string): ...
def tokenize(source, jsx: bool = ..., dotted: bool = ..., template_string: bool = ...) -> None: ...



from typing import Any, ClassVar, Tuple
from typing_extensions import Literal

from docutils import parsers
from docutils.parsers.rst import states

class Parser(parsers.Parser):
    config_section_dependencies: ClassVar[Tuple[str, ...]]
    initial_state: Literal["Body", "RFC2822Body"]
    state_classes: Any
    inliner: Any
    def __init__(self, rfc2822: bool = ..., inliner: Any | None = ...) -> None: ...

class DirectiveError(Exception):
    level: Any
    msg: str
    def __init__(self, level: Any, message: str) -> None: ...

class Directive:
    def __init__(
        self,
        name: str,
        arguments: list[Any],
        options: dict[str, Any],
        content: list[str],
        lineno: int,
        content_offset: int,
        block_text: str,
        state: states.RSTState,
        state_machine: states.RSTStateMachine,
    ) -> None: ...
    def __getattr__(self, name: str) -> Any: ...  # incomplete

def convert_directive_function(directive_fn): ...



from typing import Any

class JSONEncoder(object):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    def encode(self, o: Any): ...
    def default(self, o: Any): ...
    def iterencode(self, o: Any, _one_shot: bool): ...

class JSONEncoderForHTML(JSONEncoder): ...



from typing import Any

from braintree.exceptions.authentication_error import AuthenticationError as AuthenticationError
from braintree.exceptions.authorization_error import AuthorizationError as AuthorizationError
from braintree.exceptions.not_found_error import NotFoundError as NotFoundError
from braintree.exceptions.server_error import ServerError as ServerError
from braintree.exceptions.service_unavailable_error import ServiceUnavailableError as ServiceUnavailableError
from braintree.exceptions.too_many_requests_error import TooManyRequestsError as TooManyRequestsError
from braintree.exceptions.unexpected_error import UnexpectedError as UnexpectedError
from braintree.exceptions.upgrade_required_error import UpgradeRequiredError as UpgradeRequiredError
from braintree.util.http import Http as Http

class GraphQLClient(Http):
    @staticmethod
    def raise_exception_for_graphql_error(response) -> None: ...
    graphql_headers: Any
    def __init__(self, config: Any | None = ..., environment: Any | None = ...) -> None: ...
    def query(self, definition, variables: Any | None = ..., operation_name: Any | None = ...): ...



from typing import Any

class MultipartDataGenerator:
    data: Any
    line_break: str
    boundary: Any
    chunk_size: Any
    def __init__(self, chunk_size: int = ...) -> None: ...
    def add_params(self, params) -> None: ...
    def param_header(self): ...
    def get_post_data(self): ...



from _typeshed import SupportsRead
from typing import IO, Any, Callable, Type

from .decoder import JSONDecodeError as JSONDecodeError, JSONDecoder as JSONDecoder
from .encoder import JSONEncoder as JSONEncoder

def dumps(
    obj: Any,
    *,
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: Type[JSONEncoder] | None = ...,
    indent: None | int | str = ...,
    separators: tuple[str, str] | None = ...,
    default: Callable[[Any], Any] | None = ...,
    sort_keys: bool = ...,
    **kwds: Any,
) -> str: ...
def dump(
    obj: Any,
    fp: IO[str],
    *,
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: Type[JSONEncoder] | None = ...,
    indent: None | int | str = ...,
    separators: tuple[str, str] | None = ...,
    default: Callable[[Any], Any] | None = ...,
    sort_keys: bool = ...,
    **kwds: Any,
) -> None: ...
def loads(
    s: str | bytes,
    *,
    cls: Type[JSONDecoder] | None = ...,
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    **kwds: Any,
) -> Any: ...
def load(
    fp: SupportsRead[str | bytes],
    *,
    cls: Type[JSONDecoder] | None = ...,
    object_hook: Callable[[dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = ...,
    **kwds: Any,
) -> Any: ...
def detect_encoding(b: bytes) -> str: ...  # undocumented



from typing import Any

class ResourceAttributesMixin:
    def __getattr__(self, item): ...

class Resource(ResourceAttributesMixin):
    def __init__(self, *args, **kwargs) -> None: ...
    def __call__(self, id: Any | None = ..., format: Any | None = ..., url_override: Any | None = ...): ...
    def as_raw(self): ...
    def get(self, **kwargs): ...
    def options(self, **kwargs): ...
    def head(self, **kwargs): ...
    def post(self, data: Any | None = ..., files: Any | None = ..., **kwargs): ...
    def patch(self, data: Any | None = ..., files: Any | None = ..., **kwargs): ...
    def put(self, data: Any | None = ..., files: Any | None = ..., **kwargs): ...
    def delete(self, **kwargs): ...
    def url(self): ...

class API(ResourceAttributesMixin):
    resource_class: Any
    def __init__(
        self,
        base_url: Any | None = ...,
        auth: Any | None = ...,
        format: Any | None = ...,
        append_slash: bool = ...,
        session: Any | None = ...,
        serializer: Any | None = ...,
        raw: bool = ...,
    ) -> None: ...



import sys
import types
from _typeshed import Self
from socket import socket
from typing import Any, Callable, Type

from .base_events import Server
from .events import AbstractEventLoop, BaseDefaultEventLoopPolicy, _ProtocolFactory, _SSLContext
from .selector_events import BaseSelectorEventLoop

class AbstractChildWatcher:
    def add_child_handler(self, pid: int, callback: Callable[..., Any], *args: Any) -> None: ...
    def remove_child_handler(self, pid: int) -> bool: ...
    def attach_loop(self, loop: AbstractEventLoop | None) -> None: ...
    def close(self) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(self, typ: Type[BaseException] | None, exc: BaseException | None, tb: types.TracebackType | None) -> None: ...
    if sys.version_info >= (3, 8):
        def is_active(self) -> bool: ...

class BaseChildWatcher(AbstractChildWatcher):
    def __init__(self) -> None: ...

class SafeChildWatcher(BaseChildWatcher):
    def __enter__(self: Self) -> Self: ...

class FastChildWatcher(BaseChildWatcher):
    def __enter__(self: Self) -> Self: ...

class _UnixSelectorEventLoop(BaseSelectorEventLoop):
    if sys.version_info < (3, 7):
        async def create_unix_server(
            self,
            protocol_factory: _ProtocolFactory,
            path: str | None = ...,
            *,
            sock: socket | None = ...,
            backlog: int = ...,
            ssl: _SSLContext = ...,
        ) -> Server: ...

class _UnixDefaultEventLoopPolicy(BaseDefaultEventLoopPolicy):
    def get_child_watcher(self) -> AbstractChildWatcher: ...
    def set_child_watcher(self, watcher: AbstractChildWatcher | None) -> None: ...

SelectorEventLoop = _UnixSelectorEventLoop

DefaultEventLoopPolicy = _UnixDefaultEventLoopPolicy

if sys.version_info >= (3, 8):

    from typing import Protocol
    class _Warn(Protocol):
        def __call__(
            self, message: str, category: Type[Warning] | None = ..., stacklevel: int = ..., source: Any | None = ...
        ) -> None: ...
    class MultiLoopChildWatcher(AbstractChildWatcher):
        def __enter__(self: Self) -> Self: ...
    class ThreadedChildWatcher(AbstractChildWatcher):
        def __enter__(self: Self) -> Self: ...
        def __del__(self, _warn: _Warn = ...) -> None: ...



import distutils.command.install_lib as orig

class install_lib(orig.install_lib):
    def run(self) -> None: ...
    def get_exclusions(self): ...
    def copy_tree(
        self, infile, outfile, preserve_mode: int = ..., preserve_times: int = ..., preserve_symlinks: int = ..., level: int = ...
    ): ...
    def get_outputs(self): ...



from typing import Any

from .ImageFile import ImageFile

class IcoFile:
    buf: Any
    entry: Any
    nb_items: Any
    def __init__(self, buf): ...
    def sizes(self): ...
    def getentryindex(self, size, bpp: bool = ...): ...
    def getimage(self, size, bpp: bool = ...): ...
    def frame(self, idx): ...

class IcoImageFile(ImageFile):
    format: str
    format_description: str
    @property
    def size(self): ...
    @size.setter
    def size(self, value) -> None: ...
    im: Any
    mode: Any
    def load(self) -> None: ...
    def load_seek(self) -> None: ...



from typing import Any

from stripe import api_requestor as api_requestor, error as error
from stripe.stripe_object import StripeObject as StripeObject

class APIResource(StripeObject):
    @classmethod
    def retrieve(cls, id, api_key: Any | None = ..., **params): ...
    def refresh(self): ...
    @classmethod
    def class_url(cls): ...
    def instance_url(self): ...



from typing import Any, Callable

def _count() -> int: ...

class error(Exception): ...

class LockType:
    def acquire(self, waitflag: int = ...) -> bool: ...
    def acquire_lock(self, waitflag: int = ...) -> bool: ...
    def release(self) -> None: ...
    def release_lock(self) -> None: ...
    def locked(self) -> bool: ...
    def locked_lock(self) -> bool: ...
    def __enter__(self) -> LockType: ...
    def __exit__(self, typ: Any, value: Any, traceback: Any) -> None: ...

class _local(object): ...
class _localdummy(object): ...

def start_new(function: Callable[..., Any], args: Any, kwargs: Any = ...) -> int: ...
def start_new_thread(function: Callable[..., Any], args: Any, kwargs: Any = ...) -> int: ...
def interrupt_main() -> None: ...
def exit() -> None: ...
def exit_thread() -> Any: ...
def allocate_lock() -> LockType: ...
def get_ident() -> int: ...
def stack_size(size: int = ...) -> int: ...



from typing import Any

from .ImageFile import StubImageFile

def register_handler(handler) -> None: ...

class WmfHandler:
    bbox: Any
    def open(self, im) -> None: ...
    def load(self, im): ...

class WmfStubImageFile(StubImageFile):
    format: str
    format_description: str
    def load(self, dpi: Any | None = ...) -> None: ...



"""Support for the 'attrs' library."""

import enum
import logging
from typing import Any, ClassVar, Dict, Optional, Tuple, Union

from pytype import overlay
from pytype import overlay_utils
from pytype.abstract import abstract
from pytype.abstract import abstract_utils
from pytype.abstract import function
from pytype.abstract import mixin
from pytype.overlays import classgen

log = logging.getLogger(__name__)

# type aliases for convenience
Param = overlay_utils.Param
Attribute = classgen.Attribute


class TypeSource(enum.Enum):
  """Source of an attrib's `typ` property."""
  TYPE = 1
  DEFAULT = 2
  CONVERTER = 3


class AttrOverlay(overlay.Overlay):
  """A custom overlay for the 'attr' module."""

  def __init__(self, ctx):
    member_map = {
        "attrs": Attrs.make,
        "attrib": Attrib.make,
        "s": Attrs.make,
        "dataclass": Attrs.make_dataclass,
        "ib": Attrib.make,
        "Factory": Factory.make,

        # Attr's next-gen APIs
        # See https://www.attrs.org/en/stable/api.html#next-gen

        # They accept (almost) all the same arguments as the previous APIs.
        # Strictly speaking these only exist in Python 3.6 and up, but
        # Python 3.6 is the minimum version pytype are supports now, so not
        # bothering to guard these definitions.
        "define": AttrsNextGenDefine.make,
        # These (may) have different default arguments than 'define' for the
        # "frozen" arguments, but the overlay doesn't look at that yet.
        "mutable": AttrsNextGenDefine.make,
        "frozen": AttrsNextGenDefine.make,
        "field": Attrib.make,
    }

    ast = ctx.loader.import_name("attr")
    super().__init__(ctx, "attr", member_map, ast)


class _NoChange():
  pass

# A unique sentinel value to signal not to write anything, not even the
# original value.
_NO_CHANGE = _NoChange()


class Attrs(classgen.Decorator):
  """Implements the @attr.s decorator."""

  @classmethod
  def make(cls, ctx):
    return super().make("s", ctx, "attr")

  @classmethod
  def make_dataclass(cls, ctx):
    ret = super().make("s", ctx, "attr")
    ret.partial_args["auto_attribs"] = True
    return ret

  def init_name(self, attr):
    # attrs removes leading underscores from attrib names when generating kwargs
    # for __init__.
    return attr.name.lstrip("_")

  def _handle_auto_attribs(
      self, auto_attribs: Optional[bool], unused_local_ops,
      unused_cls_name: str) -> Tuple[Union[Optional[bool], _NoChange], Any]:
    # Why _NO_CHANGE instead of passing auto_attribs?
    # Because result is going into potentially an OrderedDict, where
    # writing even the same value might have a side effect (changing ordering).
    return _NO_CHANGE, _ordering_for_auto_attrib(auto_attribs)

  def decorate(self, node, cls):
    """Processes the attrib members of a class."""
    # Collect classvars to convert them to attrs.
    new_auto_attribs, ordering = self._handle_auto_attribs(
        self.args[cls]["auto_attribs"], self.ctx.vm.local_ops.get(cls.name, ()),
        cls.name)
    if new_auto_attribs is not _NO_CHANGE:
      self.args[cls]["auto_attribs"] = new_auto_attribs
    ordered_locals = classgen.get_class_locals(
        cls.name, allow_methods=False, ordering=ordering, ctx=self.ctx)
    own_attrs = []
    for name, local in ordered_locals.items():
      typ, orig = local.get_type(node, name), local.orig
      if is_attrib(orig):
        attrib = orig.data[0]
        attr = Attribute(
            name=name, typ=None, init=attrib.init, init_type=attrib.init_type,
            kw_only=attrib.kw_only, default=attrib.default)
        if typ:
          if attrib.type_source == TypeSource.TYPE:
            # We cannot have both a type annotation and a type argument.
            msg = "attr.ib cannot have both a 'type' arg and a type annotation."
            self.ctx.errorlog.invalid_annotation(
                self.ctx.vm.frames, typ, details=msg)
            attr.typ = self.ctx.convert.unsolvable
          elif attrib.type_source == TypeSource.CONVERTER:
            msg = "attr.ib type has been assigned by the converter."
            self.ctx.check_annotation_type_mismatch(
                node,
                name,
                typ,
                attrib.typ.instantiate(node),
                local.stack,
                allow_none=True,
                details=msg)
            attr.typ = typ
          else:
            # cls.members[name] has already been set via a type annotation
            # Use the annotation type as the field type in all circumstances; if
            # it conflicts with the `type` or `converter` args we will raise an
            # error above, and if it is compatible but not identical we treat it
            # as the type most expressive of the code's intent.
            attr.typ = typ
        else:
          # Replace the attrib in the class dict with its type.
          attr.typ = attrib.typ
          classgen.add_member(node, cls, name, attr.typ)
          if (attrib.type_source == TypeSource.TYPE and
              isinstance(cls, abstract.InterpreterClass)):
            # Add the attrib to the class's __annotations__ dict.
            annotations_dict = classgen.get_or_create_annotations_dict(
                cls.members, self.ctx)
            annotations_dict.annotated_locals[name] = abstract_utils.Local(
                node, None, attrib.typ, orig, self.ctx)
        # TODO(mdemello): This will raise a confusing 'annotation-type-mismatch'
        # error even if the type has been set via the type or converter arg
        # rather than a type annotation. We need a more general 'type-mismatch'
        # error to cover overlays that do runtime type checking. We will work
        # around it with a footnote for now.
        msg = ("Note: The 'assignment' here is the 'default' or 'factory' arg,"
               " which conflicts with the field type (set via annotation or a"
               " 'type' or 'converter' arg).")
        self.ctx.check_annotation_type_mismatch(
            node,
            attr.name,
            attr.typ,
            attr.default,
            local.stack,
            allow_none=True,
            details=msg)
        own_attrs.append(attr)
      elif self.args[cls]["auto_attribs"]:
        if not match_classvar(typ):
          self.ctx.check_annotation_type_mismatch(
              node, name, typ, orig, local.stack, allow_none=True)
          attr = Attribute(
              name=name, typ=typ, init=True, kw_only=False, default=orig)
          if not orig:
            classgen.add_member(node, cls, name, typ)
          own_attrs.append(attr)

    cls.record_attr_ordering(own_attrs)
    attrs = cls.compute_attr_metadata(own_attrs, "attr.s")

    # Add an __init__.
    # If the "init" parameter of decorator is False, instead an
    # __attrs_init__ function is generated, which is what would have been
    # generated for __init__ if the init parameter was True.
    # This logic was added in attrs version 21.1.0
    init_method_name = (
        "__init__" if self.args[cls]["init"] else "__attrs_init__")
    init_method = self.make_init(node, cls, attrs, init_method_name)
    cls.members[init_method_name] = init_method

    if isinstance(cls, abstract.InterpreterClass):
      cls.decorators.append("attr.s")
      # Fix up type parameters in methods added by the decorator.
      cls.update_method_type_params()

  def to_metadata(self):
    return {
        "tag": "attr.s",
        "init": self._current_args["init"],
        "kw_only": self._current_args["kw_only"],
        "auto_attribs": self._current_args["auto_attribs"]
    }

  @classmethod
  def from_metadata(cls, ctx, metadata):
    kwargs = {k: metadata[k] for k in ("init", "kw_only", "auto_attribs")}
    ret = cls.make(ctx)
    ret.set_current_args(kwargs)
    return ret


class AttrsNextGenDefine(Attrs):
  """Implements the @attr.define decorator.

  See https://www.attrs.org/en/stable/api.html#next-generation-apis
  """

  # Override the default arguments.
  _DEFAULT_ARGS: ClassVar[Dict[str, Any]] = {
      # Entries from Decorator._DEFAULT_ARGS
      "init": True,
      "kw_only": False,
      # Deviations from @attr.s
      "auto_attribs": None,
      # The overlay doesn't pay attention to these yet, so declaring these
      # deviations doesn't do much. Here in case in the future the overlay does.
      "slots": True,
      "weakref_slots": True,
      "auto_exc": True,
      "auto_detect": True
  }

  @classmethod
  def make(cls, ctx):
    # Bypass Attrs's make; go straight to its superclass.
    return super(Attrs, cls).make("define", ctx, "attr")  # pylint: disable=bad-super-call

  def _handle_auto_attribs(self, auto_attribs, local_ops, cls_name):
    if auto_attribs is not None:
      return super()._handle_auto_attribs(auto_attribs, local_ops, cls_name)
    is_annotated = {}
    for op in local_ops:
      local = self.ctx.vm.annotated_locals[cls_name][op.name]
      if not classgen.is_relevant_class_local(local, op.name, False):
        continue
      if op.name not in is_annotated:
        is_annotated[op.name] = op.is_annotate()
      elif op.is_annotate():
        is_annotated[op.name] = True
    all_annotated = all(is_annotated.values())
    return all_annotated, _ordering_for_auto_attrib(all_annotated)


class AttribInstance(abstract.SimpleValue, mixin.HasSlots):
  """Return value of an attr.ib() call."""

  def __init__(self, ctx, typ, type_source, init, init_type, kw_only, default):
    super().__init__("attrib", ctx)
    mixin.HasSlots.init_mixin(self)
    self.typ = typ
    self.type_source = type_source
    self.init = init
    self.init_type = init_type
    self.kw_only = kw_only
    self.default = default
    # TODO(rechen): attr.ib() returns an instance of attr._make._CountingAttr.
    self.cls = ctx.convert.unsolvable
    self.set_slot("default", self.default_slot)
    self.set_slot("validator", self.validator_slot)

  def default_slot(self, node, default):
    # If the default is a method, call it and use its return type.
    fn = default.data[0]
    # TODO(mdemello): it is not clear what to use for self in fn_args; using
    # fn.cls.instantiate(node) is fraught because we are in the process of
    # constructing the class. If fn does not use `self` setting self=Any will
    # make no difference; if it does use `self` we might as well fall back to a
    # return type of `Any` rather than raising attribute errors in cases like
    # class A:
    #   x = attr.ib(default=42)
    #   y = attr.ib()
    #   @y.default
    #   def _y(self):
    #     return self.x
    #
    # The correct thing to do would probably be to defer inference if we see a
    # default method, then infer all the method-based defaults after the class
    # is fully constructed. The workaround is simply to use type annotations,
    # which users should ideally be doing anyway.
    self_var = self.ctx.new_unsolvable(node)
    fn_args = function.Args(posargs=(self_var,))
    node, default_var = fn.call(node, default.bindings[0], fn_args)
    self.default = default_var
    # If we don't have a type, set the type from the default type
    if not self.type_source:
      self.typ = get_type_from_default(default_var, self.ctx)
      self.type_source = TypeSource.DEFAULT
    # Return the original decorated method so we don't lose it.
    return node, default

  def validator_slot(self, node, validator):
    return node, validator

  def to_metadata(self):
    type_source = self.type_source and self.type_source.name
    return {
        "tag": "attr.ib",
        "init": self.init,
        "kw_only": self.kw_only,
        "type_source": type_source,
        "default": self.default is not None
    }

  @classmethod
  def from_metadata(cls, ctx, node, typ, metadata):
    init = metadata["init"]
    kw_only = metadata["kw_only"]
    type_source = metadata["type_source"]
    if type_source:
      # Convert string back to enum
      type_source = TypeSource[type_source]
    default = ctx.new_unsolvable(node) if metadata["default"] else None
    return cls(ctx, typ, type_source, init, None, kw_only, default)


class Attrib(classgen.FieldConstructor):
  """Implements attr.ib."""

  @classmethod
  def make(cls, ctx):
    return super().make("ib", ctx, "attr")

  def _match_and_discard_args(self, node, funcb, args):
    """Discard invalid args so that we can still construct an attrib."""
    func = funcb.data
    args, errors = function.match_all_args(self.ctx, node, func, args)
    # Raise the error, but continue constructing the attrib, so that if we
    # have something like
    #   x = attr.ib(<bad args>)
    # we still proceed with the rest of the analysis with x in the list of
    # attribs, and with our best guess for typing information.
    for e, name, _ in errors:
      self.ctx.errorlog.invalid_function_call(self.ctx.vm.stack(func), e)
      # The presence of 'factory' or 'default' means we need to preserve the
      # fact that the attrib was intended to have a default, otherwise we might
      # get an invalid sig for __init__ with a non-default param following a
      # default param.
      if name != "default":
        args = args.delete_namedarg(name)
      if name == "factory":
        args = args.replace_namedarg("default", self.ctx.new_unsolvable(node))
    return args

  def call(self, node, funcb, args):
    """Returns a type corresponding to an attr."""
    args = args.simplify(node, self.ctx)
    args = self._match_and_discard_args(node, funcb, args)
    node, default_var = self._get_default_var(node, args)
    type_var = args.namedargs.get("type")
    init = self.get_kwarg(args, "init", True)
    kw_only = self.get_kwarg(args, "kw_only", False)
    conv_in, conv_out = self._get_converter_types(node, args)

    if type_var:
      type_source = TypeSource.TYPE
      allowed_type_params = (
          self.ctx.vm.frame.type_params
          |
          self.ctx.annotation_utils.get_callable_type_parameter_names(type_var))
      typ = self.ctx.annotation_utils.extract_annotation(
          node,
          type_var,
          "attr.ib",
          self.ctx.vm.simple_stack(),
          allowed_type_params=allowed_type_params)
    elif default_var:
      type_source = TypeSource.DEFAULT
      typ = get_type_from_default(default_var, self.ctx)
    else:
      type_source = None
      typ = self.ctx.convert.unsolvable

    if conv_out:
      # If a converter’s first argument has a type annotation, that type will
      # appear in the signature for __init__. When setting the field type, treat
      # args as type > converter > default
      init_type = conv_in or self.ctx.convert.unsolvable
      if type_source == TypeSource.TYPE:
        msg = ("The type annotation and assignment are set by the "
               "'type' and 'converter' args respectively.")
        self.ctx.check_annotation_type_mismatch(
            node,
            "attr.ib",
            typ,
            conv_out.instantiate(node),
            self.ctx.vm.simple_stack(),
            allow_none=True,
            details=msg)
      else:
        type_source = TypeSource.CONVERTER
        typ = conv_out
    else:
      init_type = None

    ret = AttribInstance(self.ctx, typ, type_source, init, init_type, kw_only,
                         default_var).to_variable(node)
    return node, ret

  @property
  def sig(self):
    return self.signatures[0].signature

  def _get_converter_sig(self, converter, args):
    """Return the first signature with a single argument."""
    def valid_arity(sig):
      return (sig.mandatory_param_count() <= 1 and
              (sig.maximum_param_count() is None or
               sig.maximum_param_count() >= 1))
    sigs = function.get_signatures(converter)
    valid_sigs = list(filter(valid_arity, sigs))
    if not valid_sigs:
      anyt = self.ctx.convert.unsolvable
      wanted_type = abstract.CallableClass(
          self.ctx.convert.name_to_value("typing.Callable"), {
              0: anyt,
              abstract_utils.ARGS: anyt,
              abstract_utils.RET: anyt
          }, self.ctx)
      bad_param = function.BadParam("converter", wanted_type, None)
      raise function.WrongArgTypes(self.sig, args, self.ctx, bad_param)
    return valid_sigs[0]

  def _call_converter_function(self, node, converter_var, args):
    """Run converter and return the input and return types."""
    binding = converter_var.bindings[0]
    fn = binding.data
    sig = self._get_converter_sig(fn, args)
    if sig.param_names and sig.param_names[0] in sig.annotations:
      input_type = sig.annotations[sig.param_names[0]]
    else:
      input_type = self.ctx.convert.unsolvable
    if sig.has_return_annotation:
      return_type = sig.annotations["return"]
    else:
      fn_args = function.Args(posargs=(input_type.instantiate(node),))
      node, ret_var = fn.call(node, binding, fn_args)
      return_type = self.ctx.convert.merge_classes(ret_var.data)
    return input_type, return_type

  def _get_converter_types(self, node, args):
    converter_var = args.namedargs.get("converter")
    if not converter_var:
      return None, None
    converter = converter_var.data[0]
    if isinstance(converter, abstract.Class):
      # If the converter is a class, set the field type to the class and the
      # init type to Any.
      # TODO(b/135553563): Check that converter.__init__ takes one argument and
      # get its type.
      return self.ctx.convert.unsolvable, converter
    elif abstract_utils.is_callable(converter):
      return self._call_converter_function(node, converter_var, args)
    else:
      return None, None

  def _get_default_var(self, node, args):
    if "default" in args.namedargs and "factory" in args.namedargs:
      # attr.ib(factory=x) is syntactic sugar for attr.ib(default=Factory(x)).
      raise function.DuplicateKeyword(self.sig, args, self.ctx, "default")
    elif "default" in args.namedargs:
      default_var = args.namedargs["default"]
    elif "factory" in args.namedargs:
      mod = self.ctx.vm.import_module("attr", "attr", 0)
      node, attr = self.ctx.attribute_handler.get_attribute(
          node, mod, "Factory")
      # We know there is only one value because Factory is in the overlay.
      factory, = attr.data
      factory_args = function.Args(posargs=(args.namedargs["factory"],))
      node, default_var = factory.call(node, attr.bindings[0], factory_args)
    else:
      default_var = None
    return node, default_var


def _ordering_for_auto_attrib(auto_attrib):
  return (classgen.Ordering.FIRST_ANNOTATE if auto_attrib
          else classgen.Ordering.LAST_ASSIGN)


def is_attrib(var):
  return var and isinstance(var.data[0], AttribInstance)


def match_classvar(typ):
  """Unpack the type parameter from ClassVar[T]."""
  return abstract_utils.match_type_container(typ, "typing.ClassVar")


def get_type_from_default(default_var, ctx):
  """Get the type of an attribute from its default value."""
  if default_var.data == [ctx.convert.none]:
    # A default of None doesn't give us any information about the actual type.
    return ctx.convert.unsolvable
  typ = ctx.convert.merge_classes(default_var.data)
  if typ == ctx.convert.empty:
    return ctx.convert.unsolvable
  elif isinstance(typ, abstract.TupleClass) and not typ.tuple_length:
    # The type of an attribute whose default is an empty tuple should be
    # Tuple[Any, ...], not Tuple[()].
    return ctx.convert.tuple_type
  return typ


class Factory(abstract.PyTDFunction):
  """Implementation of attr.Factory."""

  @classmethod
  def make(cls, ctx):
    return super().make("Factory", ctx, "attr")



from abc import ABC
from typing import Any

class Transition(ABC):
    def dict_as_string(self) -> None: ...

class SplitTransition(Transition):
    dimension: Any
    direction: Any
    def __init__(self, dimension, direction) -> None: ...
    def dict_as_string(self): ...

class BlindsTransition(Transition):
    dimension: Any
    def __init__(self, dimension) -> None: ...
    def dict_as_string(self): ...

class BoxTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class WipeTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class DissolveTransition(Transition):
    def dict_as_string(self): ...

class GlitterTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class FlyTransition(Transition):
    dimension: Any
    direction: Any
    def __init__(self, dimension, direction: Any | None = ...) -> None: ...
    def dict_as_string(self): ...

class PushTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class CoverTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class UncoverTransition(Transition):
    direction: Any
    def __init__(self, direction) -> None: ...
    def dict_as_string(self): ...

class FadeTransition(Transition):
    def dict_as_string(self): ...



from _typeshed import Self, SupportsRead
from typing import Any, Sequence, Type

from .builder import TreeBuilder
from .element import PageElement, SoupStrainer, Tag
from .formatter import Formatter

class GuessedAtParserWarning(UserWarning): ...
class MarkupResemblesLocatorWarning(UserWarning): ...

class BeautifulSoup(Tag):
    ROOT_TAG_NAME: str
    DEFAULT_BUILDER_FEATURES: list[str]
    ASCII_SPACES: str
    NO_PARSER_SPECIFIED_WARNING: str
    element_classes: Any
    builder: TreeBuilder
    is_xml: bool
    known_xml: bool
    parse_only: SoupStrainer | None
    markup: str
    def __init__(
        self,
        markup: str | bytes | SupportsRead[str] | SupportsRead[bytes] = ...,
        features: str | Sequence[str] | None = ...,
        builder: TreeBuilder | Type[TreeBuilder] | None = ...,
        parse_only: SoupStrainer | None = ...,
        from_encoding: str | None = ...,
        exclude_encodings: Sequence[str] | None = ...,
        element_classes: dict[Type[PageElement], Type[Any]] | None = ...,
        **kwargs,
    ) -> None: ...
    def __copy__(self: Self) -> Self: ...
    hidden: bool
    current_data: Any
    currentTag: Any
    tagStack: Any
    open_tag_counter: Any
    preserve_whitespace_tag_stack: Any
    string_container_stack: Any
    def reset(self) -> None: ...
    def new_tag(
        self,
        name,
        namespace: Any | None = ...,
        nsprefix: Any | None = ...,
        attrs=...,
        sourceline: Any | None = ...,
        sourcepos: Any | None = ...,
        **kwattrs,
    ) -> Tag: ...
    def string_container(self, base_class: Any | None = ...): ...
    def new_string(self, s, subclass: Any | None = ...): ...
    def insert_before(self, *args) -> None: ...
    def insert_after(self, *args) -> None: ...
    def popTag(self): ...
    def pushTag(self, tag) -> None: ...
    def endData(self, containerClass: Any | None = ...) -> None: ...
    def object_was_parsed(self, o, parent: Any | None = ..., most_recent_element: Any | None = ...) -> None: ...
    def handle_starttag(self, name, namespace, nsprefix, attrs, sourceline: Any | None = ..., sourcepos: Any | None = ...): ...
    def handle_endtag(self, name, nsprefix: Any | None = ...) -> None: ...
    def handle_data(self, data) -> None: ...
    def decode(self, pretty_print: bool = ..., eventual_encoding: str = ..., formatter: str | Formatter = ...): ...  # type: ignore  # missing some arguments

class BeautifulStoneSoup(BeautifulSoup): ...
class StopParsing(Exception): ...
class FeatureNotFound(ValueError): ...



import sys

if sys.version_info >= (3, 0):
    unicode = str
    unichr = chr
    basestring = str
    interactive_prompt = input
    from html.parser import HTMLParser as HTMLParser
    from io import StringIO as StringIO
else:
    unicode = unicode
    unichr = unichr
    basestring = basestring
    interactive_prompt = raw_input
    from StringIO import StringIO as StringIO

    from HTMLParser import HTMLParser as HTMLParser

def coerce_string(value): ...
def is_string(value): ...
def is_unicode(value): ...
def on_macos(): ...
def on_windows(): ...



import datetime
import uuid
from typing import IO, Any, Callable, Generic, Iterable, Optional, Sequence, Text, Tuple as _PyTuple, Type, TypeVar, Union

from click.core import Context, Parameter, _ConvertibleType, _ParamType

ParamType = _ParamType

class BoolParamType(ParamType):
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> bool: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> bool: ...

class CompositeParamType(ParamType):
    arity: int

class Choice(ParamType):
    choices: Iterable[str]
    case_sensitive: bool
    def __init__(self, choices: Iterable[str], case_sensitive: bool = ...) -> None: ...

class DateTime(ParamType):
    formats: Sequence[str]
    def __init__(self, formats: Sequence[str] | None = ...) -> None: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> datetime.datetime: ...

class FloatParamType(ParamType):
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> float: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> float: ...

class FloatRange(FloatParamType):
    min: float | None
    max: float | None
    clamp: bool
    def __init__(self, min: float | None = ..., max: float | None = ..., clamp: bool = ...) -> None: ...

class File(ParamType):
    mode: str
    encoding: str | None
    errors: str | None
    lazy: bool | None
    atomic: bool
    def __init__(
        self,
        mode: Text = ...,
        encoding: str | None = ...,
        errors: str | None = ...,
        lazy: bool | None = ...,
        atomic: bool | None = ...,
    ) -> None: ...
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> IO[Any]: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> IO[Any]: ...
    def resolve_lazy_flag(self, value: str) -> bool: ...

_F = TypeVar("_F")  # result of the function
_Func = Callable[[Optional[str]], _F]

class FuncParamType(ParamType, Generic[_F]):
    func: _Func[_F]
    def __init__(self, func: _Func[_F]) -> None: ...
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> _F: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> _F: ...

class IntParamType(ParamType):
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> int: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> int: ...

class IntRange(IntParamType):
    min: int | None
    max: int | None
    clamp: bool
    def __init__(self, min: int | None = ..., max: int | None = ..., clamp: bool = ...) -> None: ...

_PathType = TypeVar("_PathType", str, bytes)
_PathTypeBound = Union[Type[str], Type[bytes]]

class Path(ParamType):
    exists: bool
    file_okay: bool
    dir_okay: bool
    writable: bool
    readable: bool
    resolve_path: bool
    allow_dash: bool
    type: _PathTypeBound | None
    def __init__(
        self,
        exists: bool = ...,
        file_okay: bool = ...,
        dir_okay: bool = ...,
        writable: bool = ...,
        readable: bool = ...,
        resolve_path: bool = ...,
        allow_dash: bool = ...,
        path_type: Type[_PathType] | None = ...,
    ) -> None: ...
    def coerce_path_result(self, rv: str | bytes) -> _PathType: ...
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> _PathType: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> _PathType: ...

class StringParamType(ParamType):
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> str: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> str: ...

class Tuple(CompositeParamType):
    types: list[ParamType]
    def __init__(self, types: Iterable[Any]) -> None: ...
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> Tuple: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> Tuple: ...

class UnprocessedParamType(ParamType): ...

class UUIDParameterType(ParamType):
    def __call__(self, value: str | None, param: Parameter | None = ..., ctx: Context | None = ...) -> uuid.UUID: ...
    def convert(self, value: str, param: Parameter | None, ctx: Context | None) -> uuid.UUID: ...

def convert_type(ty: _ConvertibleType | None, default: Any | None = ...) -> ParamType: ...

# parameter type shortcuts

BOOL: BoolParamType
FLOAT: FloatParamType
INT: IntParamType
STRING: StringParamType
UNPROCESSED: UnprocessedParamType
UUID: UUIDParameterType



from _typeshed import SupportsRead
from typing import IO, Any, Callable, Dict, List, Text, Tuple, Type

def dumps(
    obj: Any,
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: Type[JSONEncoder] | None = ...,
    indent: int | None = ...,
    separators: Tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    sort_keys: bool = ...,
    **kwds: Any,
) -> str: ...
def dump(
    obj: Any,
    fp: IO[str] | IO[Text],
    skipkeys: bool = ...,
    ensure_ascii: bool = ...,
    check_circular: bool = ...,
    allow_nan: bool = ...,
    cls: Type[JSONEncoder] | None = ...,
    indent: int | None = ...,
    separators: Tuple[str, str] | None = ...,
    encoding: str = ...,
    default: Callable[[Any], Any] | None = ...,
    sort_keys: bool = ...,
    **kwds: Any,
) -> None: ...
def loads(
    s: Text | bytes,
    encoding: Any = ...,
    cls: Type[JSONDecoder] | None = ...,
    object_hook: Callable[[Dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[List[Tuple[Any, Any]]], Any] | None = ...,
    **kwds: Any,
) -> Any: ...
def load(
    fp: SupportsRead[Text | bytes],
    encoding: str | None = ...,
    cls: Type[JSONDecoder] | None = ...,
    object_hook: Callable[[Dict[Any, Any]], Any] | None = ...,
    parse_float: Callable[[str], Any] | None = ...,
    parse_int: Callable[[str], Any] | None = ...,
    parse_constant: Callable[[str], Any] | None = ...,
    object_pairs_hook: Callable[[List[Tuple[Any, Any]]], Any] | None = ...,
    **kwds: Any,
) -> Any: ...

class JSONDecoder(object):
    def __init__(
        self,
        encoding: Text | bytes = ...,
        object_hook: Callable[..., Any] = ...,
        parse_float: Callable[[str], float] = ...,
        parse_int: Callable[[str], int] = ...,
        parse_constant: Callable[[str], Any] = ...,
        strict: bool = ...,
        object_pairs_hook: Callable[..., Any] = ...,
    ) -> None: ...
    def decode(self, s: Text | bytes, _w: Any = ...) -> Any: ...
    def raw_decode(self, s: Text | bytes, idx: int = ...) -> Tuple[Any, Any]: ...

class JSONEncoder(object):
    item_separator: str
    key_separator: str
    skipkeys: bool
    ensure_ascii: bool
    check_circular: bool
    allow_nan: bool
    sort_keys: bool
    indent: int | None
    def __init__(
        self,
        skipkeys: bool = ...,
        ensure_ascii: bool = ...,
        check_circular: bool = ...,
        allow_nan: bool = ...,
        sort_keys: bool = ...,
        indent: int | None = ...,
        separators: Tuple[Text | bytes, Text | bytes] = ...,
        encoding: Text | bytes = ...,
        default: Callable[..., Any] = ...,
    ) -> None: ...
    def default(self, o: Any) -> Any: ...
    def encode(self, o: Any) -> str: ...
    def iterencode(self, o: Any, _one_shot: bool = ...) -> str: ...



from collections.abc import Callable
from typing import Any

__version__: str

def dumps(__obj: Any, default: Callable[[Any], Any] | None = ..., option: int | None = ...) -> bytes: ...
def loads(__obj: bytes | str) -> Any: ...

class JSONDecodeError(ValueError): ...
class JSONEncodeError(TypeError): ...

OPT_APPEND_NEWLINE: int
OPT_INDENT_2: int
OPT_NAIVE_UTC: int
OPT_NON_STR_KEYS: int
OPT_OMIT_MICROSECONDS: int
OPT_PASSTHROUGH_DATACLASS: int
OPT_PASSTHROUGH_DATETIME: int
OPT_PASSTHROUGH_SUBCLASS: int
OPT_SERIALIZE_DATACLASS: int
OPT_SERIALIZE_NUMPY: int
OPT_SERIALIZE_UUID: int
OPT_SORT_KEYS: int
OPT_STRICT_INTEGER: int
OPT_UTC_Z: int



from typing import Any

def register(viewer, order: int = ...) -> None: ...
def show(image, title: Any | None = ..., **options): ...

class Viewer:
    def show(self, image, **options): ...
    format: Any
    options: Any
    def get_format(self, image): ...
    def get_command(self, file, **options) -> None: ...
    def save_image(self, image): ...
    def show_image(self, image, **options): ...
    def show_file(self, file, **options): ...

class WindowsViewer(Viewer):
    format: str
    options: Any
    def get_command(self, file, **options): ...

class MacViewer(Viewer):
    format: str
    options: Any
    def get_command(self, file, **options): ...
    def show_file(self, file, **options): ...

class UnixViewer(Viewer):
    format: str
    options: Any
    def get_command(self, file, **options): ...
    def show_file(self, file, **options): ...

class DisplayViewer(UnixViewer):
    def get_command_ex(self, file, **options): ...

class GmDisplayViewer(UnixViewer):
    def get_command_ex(self, file, **options): ...

class EogViewer(UnixViewer):
    def get_command_ex(self, file, **options): ...

class XVViewer(UnixViewer):
    def get_command_ex(self, file, title: Any | None = ..., **options): ...

class IPythonViewer(Viewer):
    def show_image(self, image, **options): ...



from typing import Any
def f1(a, b : int):
    pass

def f2(a, b) -> int:
    pass

def f3(a):
    # type: (Any) -> r3
    pass

def f4(a, b):
    # type: (e4, Any) -> None
    pass

def f5(a):
    # type: (Any) -> Any
    pass



import sys
from typing import Any, Dict, Generic, Iterable, Mapping, TypeVar, Union, overload

if sys.version_info >= (3, 9):
    from types import GenericAlias

_DataType = Union[str, Mapping[str, Union[str, Morsel[Any]]]]
_T = TypeVar("_T")

@overload
def _quote(str: None) -> None: ...
@overload
def _quote(str: str) -> str: ...
@overload
def _unquote(str: None) -> None: ...
@overload
def _unquote(str: str) -> str: ...

class CookieError(Exception): ...

class Morsel(Dict[str, Any], Generic[_T]):
    value: str
    coded_value: _T
    key: str
    def __init__(self) -> None: ...
    if sys.version_info >= (3, 7):
        def set(self, key: str, val: str, coded_val: _T) -> None: ...
    else:
        def set(self, key: str, val: str, coded_val: _T, LegalChars: str = ...) -> None: ...
    def setdefault(self, key: str, val: str | None = ...) -> str: ...
    # The dict update can also get a keywords argument so this is incompatible
    @overload  # type: ignore[override]
    def update(self, values: Mapping[str, str]) -> None: ...
    @overload
    def update(self, values: Iterable[tuple[str, str]]) -> None: ...
    def isReservedKey(self, K: str) -> bool: ...
    def output(self, attrs: list[str] | None = ..., header: str = ...) -> str: ...
    def js_output(self, attrs: list[str] | None = ...) -> str: ...
    def OutputString(self, attrs: list[str] | None = ...) -> str: ...
    if sys.version_info >= (3, 9):
        def __class_getitem__(cls, item: Any) -> GenericAlias: ...

class BaseCookie(Dict[str, Morsel[_T]], Generic[_T]):
    def __init__(self, input: _DataType | None = ...) -> None: ...
    def value_decode(self, val: str) -> _T: ...
    def value_encode(self, val: _T) -> str: ...
    def output(self, attrs: list[str] | None = ..., header: str = ..., sep: str = ...) -> str: ...
    def js_output(self, attrs: list[str] | None = ...) -> str: ...
    def load(self, rawdata: _DataType) -> None: ...
    def __setitem__(self, key: str, value: str | Morsel[_T]) -> None: ...

class SimpleCookie(BaseCookie[_T], Generic[_T]): ...



from typing import Dict, Mapping, Sequence, Union

_Cap = Dict[str, Union[str, int]]

def findmatch(
    caps: Mapping[str, list[_Cap]], MIMEtype: str, key: str = ..., filename: str = ..., plist: Sequence[str] = ...
) -> tuple[str | None, _Cap | None]: ...
def getcaps() -> dict[str, list[_Cap]]: ...



class ErrorCodes:
    class Address:
        CannotBeBlank: str
        CompanyIsInvalid: str
        CompanyIsTooLong: str
        CountryCodeAlpha2IsNotAccepted: str
        CountryCodeAlpha3IsNotAccepted: str
        CountryCodeNumericIsNotAccepted: str
        CountryNameIsNotAccepted: str
        ExtendedAddressIsInvalid: str
        ExtendedAddressIsTooLong: str
        FirstNameIsInvalid: str
        FirstNameIsTooLong: str
        InconsistentCountry: str
        IsInvalid: str
        LastNameIsInvalid: str
        LastNameIsTooLong: str
        LocalityIsInvalid: str
        LocalityIsTooLong: str
        PostalCodeInvalidCharacters: str
        PostalCodeIsInvalid: str
        PostalCodeIsRequired: str
        PostalCodeIsRequiredForCardBrandAndProcessor: str
        PostalCodeIsTooLong: str
        RegionIsInvalid: str
        RegionIsTooLong: str
        StateIsInvalidForSellerProtection: str
        StreetAddressIsInvalid: str
        StreetAddressIsRequired: str
        StreetAddressIsTooLong: str
        TooManyAddressesPerCustomer: str
    class ApplePay:
        ApplePayCardsAreNotAccepted: str
        CustomerIdIsRequiredForVaulting: str
        TokenIsInUse: str
        PaymentMethodNonceConsumed: str
        PaymentMethodNonceUnknown: str
        PaymentMethodNonceLocked: str
        PaymentMethodNonceCardTypeIsNotAccepted: str
        CannotUpdateApplePayCardUsingPaymentMethodNonce: str
        NumberIsRequired: str
        ExpirationMonthIsRequired: str
        ExpirationYearIsRequired: str
        CryptogramIsRequired: str
        DecryptionFailed: str
        Disabled: str
        MerchantNotConfigured: str
        MerchantKeysAlreadyConfigured: str
        MerchantKeysNotConfigured: str
        CertificateInvalid: str
        CertificateMismatch: str
        InvalidToken: str
        PrivateKeyMismatch: str
        KeyMismatchStoringCertificate: str
    class AuthorizationFingerprint:
        MissingFingerprint: str
        InvalidFormat: str
        SignatureRevoked: str
        InvalidCreatedAt: str
        InvalidPublicKey: str
        InvalidSignature: str
        OptionsNotAllowedWithoutCustomer: str
    class ClientToken:
        MakeDefaultRequiresCustomerId: str
        VerifyCardRequiresCustomerId: str
        FailOnDuplicatePaymentMethodRequiresCustomerId: str
        CustomerDoesNotExist: str
        ProxyMerchantDoesNotExist: str
        UnsupportedVersion: str
        MerchantAccountDoesNotExist: str
    class CreditCard:
        BillingAddressConflict: str
        BillingAddressFormatIsInvalid: str
        BillingAddressIdIsInvalid: str
        CannotUpdateCardUsingPaymentMethodNonce: str
        CardholderNameIsTooLong: str
        CreditCardTypeIsNotAccepted: str
        CreditCardTypeIsNotAcceptedBySubscriptionMerchantAccount: str
        CustomerIdIsInvalid: str
        CustomerIdIsRequired: str
        CvvIsInvalid: str
        CvvIsRequired: str
        CvvVerificationFailed: str
        DuplicateCardExists: str
        ExpirationDateConflict: str
        ExpirationDateIsInvalid: str
        ExpirationDateIsRequired: str
        ExpirationDateYearIsInvalid: str
        ExpirationMonthIsInvalid: str
        ExpirationYearIsInvalid: str
        InvalidParamsForCreditCardUpdate: str
        InvalidVenmoSDKPaymentMethodCode: str
        NumberHasInvalidLength: str
        NumberLengthIsInvalid: str
        NumberIsInvalid: str
        NumberIsProhibited: str
        NumberIsRequired: str
        NumberMustBeTestNumber: str
        PaymentMethodConflict: str
        PaymentMethodIsNotACreditCard: str
        PaymentMethodNonceCardTypeIsNotAccepted: str
        PaymentMethodNonceConsumed: str
        PaymentMethodNonceLocked: str
        PaymentMethodNonceUnknown: str
        PostalCodeVerificationFailed: str
        TokenInvalid: str
        TokenFormatIsInvalid: str
        TokenIsInUse: str
        TokenIsNotAllowed: str
        TokenIsRequired: str
        TokenIsTooLong: str
        VenmoSDKPaymentMethodCodeCardTypeIsNotAccepted: str
        VerificationNotSupportedOnThisMerchantAccount: str
        VerificationAccountTypeIsInvald: str
        VerificationAccountTypeNotSupported: str
        class Options:
            UpdateExistingTokenIsInvalid: str
            UpdateExistingTokenNotAllowed: str
            VerificationAmountCannotBeNegative: str
            VerificationAmountFormatIsInvalid: str
            VerificationAmountIsTooLarge: str
            VerificationAmountNotSupportedByProcessor: str
            VerificationMerchantAccountIdIsInvalid: str
            VerificationMerchantAccountIsForbidden: str
            VerificationMerchantAccountIsSuspended: str
            VerificationMerchantAccountCannotBeSubMerchantAccount: str
    class Customer:
        CompanyIsTooLong: str
        CustomFieldIsInvalid: str
        CustomFieldIsTooLong: str
        EmailIsInvalid: str
        EmailFormatIsInvalid: str
        EmailIsRequired: str
        EmailIsTooLong: str
        FaxIsTooLong: str
        FirstNameIsTooLong: str
        IdIsInUse: str
        IdIsInvalid: str
        IdIsNotAllowed: str
        IdIsRequired: str
        IdIsTooLong: str
        LastNameIsTooLong: str
        PhoneIsTooLong: str
        VaultedPaymentInstrumentNonceBelongsToDifferentCustomer: str
        WebsiteIsInvalid: str
        WebsiteFormatIsInvalid: str
        WebsiteIsTooLong: str
    class Descriptor:
        DynamicDescriptorsDisabled: str
        InternationalNameFormatIsInvalid: str
        InternationalPhoneFormatIsInvalid: str
        NameFormatIsInvalid: str
        PhoneFormatIsInvalid: str
        UrlFormatIsInvalid: str
    class Dispute:
        CanOnlyAddEvidenceToOpenDispute: str
        CanOnlyRemoveEvidenceFromOpenDispute: str
        CanOnlyAddEvidenceDocumentToDispute: str
        CanOnlyAcceptOpenDispute: str
        CanOnlyFinalizeOpenDispute: str
        CanOnlyCreateEvidenceWithValidCategory: str
        EvidenceContentDateInvalid: str
        EvidenceContentTooLong: str
        EvidenceContentARNTooLong: str
        EvidenceContentPhoneTooLong: str
        EvidenceCategoryTextOnly: str
        EvidenceCategoryDocumentOnly: str
        EvidenceCategoryNotForReasonCode: str
        EvidenceCategoryDuplicate: str
        EvidenceContentEmailInvalid: str
        DigitalGoodsMissingEvidence: str
        DigitalGoodsMissingDownloadDate: str
        NonDisputedPriorTransactionEvidenceMissingARN: str
        NonDisputedPriorTransactionEvidenceMissingDate: str
        RecurringTransactionEvidenceMissingDate: str
        RecurringTransactionEvidenceMissingARN: str
        ValidEvidenceRequiredToFinalize: str
    class DocumentUpload:
        KindIsInvalid: str
        FileIsTooLarge: str
        FileTypeIsInvalid: str
        FileIsMalformedOrEncrypted: str
        FileIsTooLong: str
        FileIsEmpty: str
    class Merchant:
        CountryCannotBeBlank: str
        CountryCodeAlpha2IsInvalid: str
        CountryCodeAlpha2IsNotAccepted: str
        CountryCodeAlpha3IsInvalid: str
        CountryCodeAlpha3IsNotAccepted: str
        CountryCodeNumericIsInvalid: str
        CountryCodeNumericIsNotAccepted: str
        CountryNameIsInvalid: str
        CountryNameIsNotAccepted: str
        CurrenciesAreInvalid: str
        EmailFormatIsInvalid: str
        EmailIsRequired: str
        InconsistentCountry: str
        PaymentMethodsAreInvalid: str
        PaymentMethodsAreNotAllowed: str
        MerchantAccountExistsForCurrency: str
        CurrencyIsRequired: str
        CurrencyIsInvalid: str
        NoMerchantAccounts: str
        MerchantAccountExistsForId: str
    class MerchantAccount:
        IdFormatIsInvalid: str
        IdIsInUse: str
        IdIsNotAllowed: str
        IdIsTooLong: str
        MasterMerchantAccountIdIsInvalid: str
        MasterMerchantAccountIdIsRequired: str
        MasterMerchantAccountMustBeActive: str
        TosAcceptedIsRequired: str
        CannotBeUpdated: str
        IdCannotBeUpdated: str
        MasterMerchantAccountIdCannotBeUpdated: str
        Declined: str
        DeclinedMasterCardMatch: str
        DeclinedOFAC: str
        DeclinedFailedKYC: str
        DeclinedSsnInvalid: str
        DeclinedSsnMatchesDeceased: str
        class ApplicantDetails:
            AccountNumberIsRequired: str
            CompanyNameIsInvalid: str
            CompanyNameIsRequiredWithTaxId: str
            DateOfBirthIsRequired: str
            Declined: str
            DeclinedMasterCardMatch: str
            DeclinedOFAC: str
            DeclinedFailedKYC: str
            DeclinedSsnInvalid: str
            DeclinedSsnMatchesDeceased: str
            EmailAddressIsInvalid: str
            FirstNameIsInvalid: str
            FirstNameIsRequired: str
            LastNameIsInvalid: str
            LastNameIsRequired: str
            PhoneIsInvalid: str
            RoutingNumberIsInvalid: str
            RoutingNumberIsRequired: str
            SsnIsInvalid: str
            TaxIdIsInvalid: str
            TaxIdIsRequiredWithCompanyName: str
            DateOfBirthIsInvalid: str
            EmailAddressIsRequired: str
            AccountNumberIsInvalid: str
            TaxIdMustBeBlank: str
            class Address:
                LocalityIsRequired: str
                PostalCodeIsInvalid: str
                PostalCodeIsRequired: str
                RegionIsRequired: str
                StreetAddressIsInvalid: str
                StreetAddressIsRequired: str
                RegionIsInvalid: str
        class Individual:
            FirstNameIsRequired: str
            LastNameIsRequired: str
            DateOfBirthIsRequired: str
            SsnIsInvalid: str
            EmailAddressIsInvalid: str
            FirstNameIsInvalid: str
            LastNameIsInvalid: str
            PhoneIsInvalid: str
            DateOfBirthIsInvalid: str
            EmailAddressIsRequired: str
            class Address:
                StreetAddressIsRequired: str
                LocalityIsRequired: str
                PostalCodeIsRequired: str
                RegionIsRequired: str
                StreetAddressIsInvalid: str
                PostalCodeIsInvalid: str
                RegionIsInvalid: str
        class Business:
            DbaNameIsInvalid: str
            LegalNameIsInvalid: str
            LegalNameIsRequiredWithTaxId: str
            TaxIdIsInvalid: str
            TaxIdIsRequiredWithLegalName: str
            TaxIdMustBeBlank: str
            class Address:
                StreetAddressIsInvalid: str
                PostalCodeIsInvalid: str
                RegionIsInvalid: str
        class Funding:
            RoutingNumberIsRequired: str
            AccountNumberIsRequired: str
            RoutingNumberIsInvalid: str
            AccountNumberIsInvalid: str
            DestinationIsInvalid: str
            DestinationIsRequired: str
            EmailAddressIsInvalid: str
            EmailAddressIsRequired: str
            MobilePhoneIsInvalid: str
            MobilePhoneIsRequired: str
    class OAuth:
        InvalidGrant: str
        InvalidCredentials: str
        InvalidScope: str
        InvalidRequest: str
        UnsupportedGrantType: str
    class Verification:
        ThreeDSecureAuthenticationIdIsInvalid: str
        ThreeDSecureAuthenticationIdDoesntMatchNonceThreeDSecureAuthentication: str
        ThreeDSecureTransactionPaymentMethodDoesntMatchThreeDSecureAuthenticationPaymentMethod: str
        ThreeDSecureAuthenticationIdWithThreeDSecurePassThruIsInvalid: str
        ThreeDSecureAuthenticationFailed: str
        ThreeDSecureTokenIsInvalid: str
        ThreeDSecureVerificationDataDoesntMatchVerify: str
        MerchantAccountDoesNotSupport3DSecure: str
        MerchantAcountDoesNotMatch3DSecureMerchantAccount: str
        AmountDoesNotMatch3DSecureAmount: str
        class ThreeDSecurePassThru:
            EciFlagIsRequired: str
            EciFlagIsInvalid: str
            CavvIsRequired: str
            ThreeDSecureVersionIsRequired: str
            ThreeDSecureVersionIsInvalid: str
            AuthenticationResponseIsInvalid: str
            DirectoryResponseIsInvalid: str
            CavvAlgorithmIsInvalid: str
        class Options:
            AmountCannotBeNegative: str
            AmountFormatIsInvalid: str
            AmountIsTooLarge: str
            AmountNotSupportedByProcessor: str
            MerchantAccountIdIsInvalid: str
            MerchantAccountIsSuspended: str
            MerchantAccountIsForbidden: str
            MerchantAccountCannotBeSubMerchantAccount: str
            AccountTypeIsInvalid: str
            AccountTypeNotSupported: str
    class PaymentMethod:
        CannotForwardPaymentMethodType: str
        PaymentMethodParamsAreRequired: str
        NonceIsInvalid: str
        NonceIsRequired: str
        CustomerIdIsRequired: str
        CustomerIdIsInvalid: str
        PaymentMethodNonceConsumed: str
        PaymentMethodNonceUnknown: str
        PaymentMethodNonceLocked: str
        PaymentMethodNoLongerSupported: str
        AuthExpired: str
        CannotHaveFundingSourceWithoutAccessToken: str
        InvalidFundingSourceSelection: str
        CannotUpdatePayPalAccountUsingPaymentMethodNonce: str
        class Options:
            UsBankAccountVerificationMethodIsInvalid: str
    class PayPalAccount:
        CannotHaveBothAccessTokenAndConsentCode: str
        CannotVaultOneTimeUsePayPalAccount: str
        ConsentCodeOrAccessTokenIsRequired: str
        CustomerIdIsRequiredForVaulting: str
        InvalidParamsForPayPalAccountUpdate: str
        PayPalAccountsAreNotAccepted: str
        PayPalCommunicationError: str
        PaymentMethodNonceConsumed: str
        PaymentMethodNonceLocked: str
        PaymentMethodNonceUnknown: str
        TokenIsInUse: str
    class SettlementBatchSummary:
        CustomFieldIsInvalid: str
        SettlementDateIsInvalid: str
        SettlementDateIsRequired: str
    class SEPAMandate:
        TypeIsRequired: str
        IBANInvalidCharacter: str
        BICInvalidCharacter: str
        BICLengthIsInvalid: str
        BICUnsupportedCountry: str
        IBANUnsupportedCountry: str
        IBANInvalidFormat: str
        BillingAddressConflict: str
        BillingAddressIdIsInvalid: str
        TypeIsInvalid: str
    class EuropeBankAccount:
        BICIsRequired: str
        IBANIsRequired: str
        AccountHolderNameIsRequired: str
    class Subscription:
        BillingDayOfMonthCannotBeUpdated: str
        BillingDayOfMonthIsInvalid: str
        BillingDayOfMonthMustBeNumeric: str
        CannotAddDuplicateAddonOrDiscount: str
        CannotEditCanceledSubscription: str
        CannotEditExpiredSubscription: str
        CannotEditPriceChangingFieldsOnPastDueSubscription: str
        FirstBillingDateCannotBeInThePast: str
        FirstBillingDateCannotBeUpdated: str
        FirstBillingDateIsInvalid: str
        IdIsInUse: str
        InconsistentNumberOfBillingCycles: str
        InconsistentStartDate: str
        InvalidRequestFormat: str
        MerchantAccountDoesNotSupportInstrumentType: str
        MerchantAccountIdIsInvalid: str
        MismatchCurrencyISOCode: str
        NumberOfBillingCyclesCannotBeBlank: str
        NumberOfBillingCyclesIsTooSmall: str
        NumberOfBillingCyclesMustBeGreaterThanZero: str
        NumberOfBillingCyclesMustBeNumeric: str
        PaymentMethodNonceCardTypeIsNotAccepted: str
        PaymentMethodNonceInstrumentTypeDoesNotSupportSubscriptions: str
        PaymentMethodNonceIsInvalid: str
        PaymentMethodNonceNotAssociatedWithCustomer: str
        PaymentMethodNonceUnvaultedCardIsNotAccepted: str
        PaymentMethodTokenCardTypeIsNotAccepted: str
        PaymentMethodTokenInstrumentTypeDoesNotSupportSubscriptions: str
        PaymentMethodTokenIsInvalid: str
        PaymentMethodTokenNotAssociatedWithCustomer: str
        PlanBillingFrequencyCannotBeUpdated: str
        PlanIdIsInvalid: str
        PriceCannotBeBlank: str
        PriceFormatIsInvalid: str
        PriceIsTooLarge: str
        StatusIsCanceled: str
        TokenFormatIsInvalid: str
        TrialDurationFormatIsInvalid: str
        TrialDurationIsRequired: str
        TrialDurationUnitIsInvalid: str
        class Modification:
            AmountCannotBeBlank: str
            AmountIsInvalid: str
            AmountIsTooLarge: str
            CannotEditModificationsOnPastDueSubscription: str
            CannotUpdateAndRemove: str
            ExistingIdIsIncorrectKind: str
            ExistingIdIsInvalid: str
            ExistingIdIsRequired: str
            IdToRemoveIsIncorrectKind: str
            IdToRemoveIsNotPresent: str
            InconsistentNumberOfBillingCycles: str
            InheritedFromIdIsInvalid: str
            InheritedFromIdIsRequired: str
            Missing: str
            NumberOfBillingCyclesCannotBeBlank: str
            NumberOfBillingCyclesIsInvalid: str
            NumberOfBillingCyclesMustBeGreaterThanZero: str
            QuantityCannotBeBlank: str
            QuantityIsInvalid: str
            QuantityMustBeGreaterThanZero: str
            IdToRemoveIsInvalid: str
    class Transaction:
        AdjustmentAmountMustBeGreaterThanZero: str
        AmountCannotBeNegative: str
        AmountDoesNotMatch3DSecureAmount: str
        AmountIsInvalid: str
        AmountFormatIsInvalid: str
        AmountIsRequired: str
        AmountIsTooLarge: str
        AmountMustBeGreaterThanZero: str
        AmountNotSupportedByProcessor: str
        BillingAddressConflict: str
        BillingPhoneNumberIsInvalid: str
        CannotBeVoided: str
        CannotCancelRelease: str
        CannotCloneCredit: str
        CannotCloneMarketplaceTransaction: str
        CannotCloneTransactionWithPayPalAccount: str
        CannotCloneTransactionWithVaultCreditCard: str
        CannotCloneUnsuccessfulTransaction: str
        CannotCloneVoiceAuthorizations: str
        CannotHoldInEscrow: str
        CannotPartiallyRefundEscrowedTransaction: str
        CannotRefundCredit: str
        CannotRefundSettlingTransaction: str
        CannotRefundUnlessSettled: str
        CannotRefundWithPendingMerchantAccount: str
        CannotRefundWithSuspendedMerchantAccount: str
        CannotReleaseFromEscrow: str
        CannotSimulateTransactionSettlement: str
        CannotSubmitForPartialSettlement: str
        CannotSubmitForSettlement: str
        CannotUpdateTransactionDetailsNotSubmittedForSettlement: str
        ChannelIsTooLong: str
        CreditCardIsRequired: str
        CustomFieldIsInvalid: str
        CustomFieldIsTooLong: str
        CustomerDefaultPaymentMethodCardTypeIsNotAccepted: str
        CustomerDoesNotHaveCreditCard: str
        CustomerIdIsInvalid: str
        DiscountAmountCannotBeNegative: str
        DiscountAmountFormatIsInvalid: str
        DiscountAmountIsTooLarge: str
        ExchangeRateQuoteIdIsTooLong: str
        FailedAuthAdjustmentAllowRetry: str
        FailedAuthAdjustmentHardDecline: str
        FinalAuthSubmitForSettlementForDifferentAmount: str
        HasAlreadyBeenRefunded: str
        LineItemsExpected: str
        MerchantAccountDoesNotMatch3DSecureMerchantAccount: str
        MerchantAccountDoesNotSupportMOTO: str
        MerchantAccountDoesNotSupportRefunds: str
        MerchantAccountIdDoesNotMatchSubscription: str
        MerchantAccountIdIsInvalid: str
        MerchantAccountIsSuspended: str
        NoNetAmountToPerformAuthAdjustment: str
        OrderIdIsTooLong: str
        PayPalAuthExpired: str
        PayPalNotEnabled: str
        PayPalVaultRecordMissingData: str
        PaymentInstrumentNotSupportedByMerchantAccount: str
        PaymentInstrumentTypeIsNotAccepted: str
        PaymentInstrumentWithExternalVaultIsInvalid: str
        PaymentMethodConflict: str
        PaymentMethodConflictWithVenmoSDK: str
        PaymentMethodDoesNotBelongToCustomer: str
        PaymentMethodDoesNotBelongToSubscription: str
        PaymentMethodNonceCardTypeIsNotAccepted: str
        PaymentMethodNonceConsumed: str
        PaymentMethodNonceHasNoValidPaymentInstrumentType: str
        PaymentMethodNonceLocked: str
        PaymentMethodNonceUnknown: str
        PaymentMethodTokenCardTypeIsNotAccepted: str
        PaymentMethodTokenIsInvalid: str
        ProcessorAuthorizationCodeCannotBeSet: str
        ProcessorAuthorizationCodeIsInvalid: str
        ProcessorDoesNotSupportAuths: str
        ProcessorDoesNotSupportAuthAdjustment: str
        ProcessorDoesNotSupportCredits: str
        ProcessorDoesNotSupportIncrementalAuth: str
        ProcessorDoesNotSupportMotoForCardType: str
        ProcessorDoesNotSupportPartialAuthReversal: str
        ProcessorDoesNotSupportPartialSettlement: str
        ProcessorDoesNotSupportUpdatingDescriptor: str
        ProcessorDoesNotSupportUpdatingOrderId: str
        ProcessorDoesNotSupportUpdatingTransactionDetails: str
        ProcessorDoesNotSupportVoiceAuthorizations: str
        ProductSkuIsInvalid: str
        PurchaseOrderNumberIsInvalid: str
        PurchaseOrderNumberIsTooLong: str
        RefundAmountIsTooLarge: str
        RefundAuthHardDeclined: str
        RefundAuthSoftDeclined: str
        ScaExemptionInvalid: str
        ServiceFeeAmountCannotBeNegative: str
        ServiceFeeAmountFormatIsInvalid: str
        ServiceFeeAmountIsTooLarge: str
        ServiceFeeAmountNotAllowedOnMasterMerchantAccount: str
        ServiceFeeIsNotAllowedOnCredits: str
        ServiceFeeNotAcceptedForPayPal: str
        SettlementAmountIsLessThanServiceFeeAmount: str
        SettlementAmountIsTooLarge: str
        ShippingAddressDoesntMatchCustomer: str
        ShippingAmountCannotBeNegative: str
        ShippingAmountFormatIsInvalid: str
        ShippingAmountIsTooLarge: str
        ShippingMethodIsInvalid: str
        ShippingPhoneNumberIsInvalid: str
        ShipsFromPostalCodeInvalidCharacters: str
        ShipsFromPostalCodeIsInvalid: str
        ShipsFromPostalCodeIsTooLong: str
        SubMerchantAccountRequiresServiceFeeAmount: str
        SubscriptionDoesNotBelongToCustomer: str
        SubscriptionIdIsInvalid: str
        SubscriptionStatusMustBePastDue: str
        TaxAmountCannotBeNegative: str
        TaxAmountFormatIsInvalid: str
        TaxAmountIsRequiredForAibSwedish: str
        TaxAmountIsTooLarge: str
        ThreeDSecureAuthenticationFailed: str
        ThreeDSecureAuthenticationIdDoesntMatchNonceThreeDSecureAuthentication: str
        ThreeDSecureAuthenticationIdIsInvalid: str
        ThreeDSecureAuthenticationIdWithThreeDSecurePassThruIsInvalid: str
        ThreeDSecureAuthenticationResponseIsInvalid: str
        ThreeDSecureCavvAlgorithmIsInvalid: str
        ThreeDSecureCavvIsRequired: str
        ThreeDSecureDirectoryResponseIsInvalid: str
        ThreeDSecureEciFlagIsInvalid: str
        ThreeDSecureEciFlagIsRequired: str
        ThreeDSecureMerchantAccountDoesNotSupportCardType: str
        ThreeDSecureTokenIsInvalid: str
        ThreeDSecureTransactionDataDoesntMatchVerify: str
        ThreeDSecureTransactionPaymentMethodDoesntMatchThreeDSecureAuthenticationPaymentMethod: str
        ThreeDSecureXidIsRequired: str
        TooManyLineItems: str
        TransactionIsNotEligibleForAdjustment: str
        TransactionMustBeInStateAuthorized: str
        TransactionSourceIsInvalid: str
        TypeIsInvalid: str
        TypeIsRequired: str
        UnsupportedVoiceAuthorization: str
        UsBankAccountNonceMustBePlaidVerified: str
        UsBankAccountNotVerified: str
        class ExternalVault:
            StatusIsInvalid: str
            StatusWithPreviousNetworkTransactionIdIsInvalid: str
            CardTypeIsInvalid: str
            PreviousNetworkTransactionIdIsInvalid: str
        class Options:
            SubmitForSettlementIsRequiredForCloning: str
            SubmitForSettlementIsRequiredForPayPalUnilateral: str
            UseBillingForShippingDisabled: str
            VaultIsDisabled: str
            class PayPal:
                CustomFieldTooLong: str
            class CreditCard:
                AccountTypeIsInvalid: str
                AccountTypeNotSupported: str
                AccountTypeDebitDoesNotSupportAuths: str
        class Industry:
            IndustryTypeIsInvalid: str
            class Lodging:
                EmptyData: str
                FolioNumberIsInvalid: str
                CheckInDateIsInvalid: str
                CheckOutDateIsInvalid: str
                CheckOutDateMustFollowCheckInDate: str
                UnknownDataField: str
                RoomRateMustBeGreaterThanZero: str
                RoomRateFormatIsInvalid: str
                RoomRateIsTooLarge: str
                RoomTaxMustBeGreaterThanZero: str
                RoomTaxFormatIsInvalid: str
                RoomTaxIsTooLarge: str
                NoShowIndicatorIsInvalid: str
                AdvancedDepositIndicatorIsInvalid: str
                FireSafetyIndicatorIsInvalid: str
                PropertyPhoneIsInvalid: str
            class TravelCruise:
                EmptyData: str
                UnknownDataField: str
                TravelPackageIsInvalid: str
                DepartureDateIsInvalid: str
                LodgingCheckInDateIsInvalid: str
                LodgingCheckOutDateIsInvalid: str
            class TravelFlight:
                EmptyData: str
                UnknownDataField: str
                CustomerCodeIsTooLong: str
                FareAmountCannotBeNegative: str
                FareAmountFormatIsInvalid: str
                FareAmountIsTooLarge: str
                FeeAmountCannotBeNegative: str
                FeeAmountFormatIsInvalid: str
                FeeAmountIsTooLarge: str
                IssuedDateFormatIsInvalid: str
                IssuingCarrierCodeIsTooLong: str
                PassengerMiddleInitialIsTooLong: str
                RestrictedTicketIsRequired: str
                TaxAmountCannotBeNegative: str
                TaxAmountFormatIsInvalid: str
                TaxAmountIsTooLarge: str
                TicketNumberIsTooLong: str
                LegsExpected: str
                TooManyLegs: str
            class Leg:
                class TravelFlight:
                    ArrivalAirportCodeIsTooLong: str
                    ArrivalTimeFormatIsInvalid: str
                    CarrierCodeIsTooLong: str
                    ConjunctionTicketIsTooLong: str
                    CouponNumberIsTooLong: str
                    DepartureAirportCodeIsTooLong: str
                    DepartureTimeFormatIsInvalid: str
                    ExchangeTicketIsTooLong: str
                    FareAmountCannotBeNegative: str
                    FareAmountFormatIsInvalid: str
                    FareAmountIsTooLarge: str
                    FareBasisCodeIsTooLong: str
                    FeeAmountCannotBeNegative: str
                    FeeAmountFormatIsInvalid: str
                    FeeAmountIsTooLarge: str
                    ServiceClassIsTooLong: str
                    TaxAmountCannotBeNegative: str
                    TaxAmountFormatIsInvalid: str
                    TaxAmountIsTooLarge: str
                    TicketNumberIsTooLong: str
            class AdditionalCharge:
                KindIsInvalid: str
                KindMustBeUnique: str
                AmountMustBeGreaterThanZero: str
                AmountFormatIsInvalid: str
                AmountIsTooLarge: str
                AmountIsRequired: str
        class LineItem:
            CommodityCodeIsTooLong: str
            DescriptionIsTooLong: str
            DiscountAmountFormatIsInvalid: str
            DiscountAmountIsTooLarge: str
            DiscountAmountCannotBeNegative: str
            KindIsInvalid: str
            KindIsRequired: str
            NameIsRequired: str
            NameIsTooLong: str
            ProductCodeIsTooLong: str
            QuantityFormatIsInvalid: str
            QuantityIsRequired: str
            QuantityIsTooLarge: str
            TotalAmountFormatIsInvalid: str
            TotalAmountIsRequired: str
            TotalAmountIsTooLarge: str
            TotalAmountMustBeGreaterThanZero: str
            UnitAmountFormatIsInvalid: str
            UnitAmountIsRequired: str
            UnitAmountIsTooLarge: str
            UnitAmountMustBeGreaterThanZero: str
            UnitOfMeasureIsTooLarge: str
            UnitTaxAmountFormatIsInvalid: str
            UnitTaxAmountIsTooLarge: str
            UnitTaxAmountCannotBeNegative: str
            TaxAmountFormatIsInvalid: str
            TaxAmountIsTooLarge: str
            TaxAmountCannotBeNegative: str
    class UsBankAccountVerification:
        NotConfirmable: str
        MustBeMicroTransfersVerification: str
        AmountsDoNotMatch: str
        TooManyConfirmationAttempts: str
        UnableToConfirmDepositAmounts: str
        InvalidDepositAmounts: str
    class RiskData:
        CustomerBrowserIsTooLong: str
        CustomerDeviceIdIsTooLong: str
        CustomerLocationZipInvalidCharacters: str
        CustomerLocationZipIsInvalid: str
        CustomerLocationZipIsTooLong: str
        CustomerTenureIsTooLong: str



from braintree.exceptions.http.connection_error import ConnectionError as ConnectionError
from braintree.exceptions.http.invalid_response_error import InvalidResponseError as InvalidResponseError
from braintree.exceptions.http.timeout_error import TimeoutError as TimeoutError



from _typeshed import IdentityFunction
from typing import Callable, Sequence, TypeVar

_T = TypeVar("_T")

def fifo_cache(maxsize: float = ..., typed: bool = ...) -> IdentityFunction: ...
def lfu_cache(maxsize: float = ..., typed: bool = ...) -> IdentityFunction: ...
def lru_cache(maxsize: float = ..., typed: bool = ...) -> IdentityFunction: ...
def mru_cache(maxsize: float = ..., typed: bool = ...) -> IdentityFunction: ...
def rr_cache(maxsize: float = ..., choice: Callable[[Sequence[_T]], _T] | None = ..., typed: bool = ...) -> IdentityFunction: ...
def ttl_cache(
    maxsize: float = ..., ttl: float = ..., timer: Callable[[], float] = ..., typed: bool = ...
) -> IdentityFunction: ...
